import json
import os
from .CloudApiBase import CloudApiBase
from ..RecordRequests import PersonatorIdentityRecordRequest


class PersonatorIdentity(CloudApiBase):

    def __init__(self, license=None):
        super().__init__("https://globalpersonator.melissadata.net", "/v1/doContactVerify")
        self.license = license or os.environ.get("MD_LICENSE")
        self.parameters["format"] = "json"
        self.post_body = None

        self.parameter_mappings = {
            "action": "act",
            "national_id": "nat",
            "first_name": "first",
            "last_name": "last",
            "full_name": "full",
            "company": "comp",
            "phone": "phone",
            "email": "email",
            "dob": "dob",
            "account_number": "accountNumber",
            "address_line_1": "a1",
            "address_line_2": "a2",
            "address_line_3": "a3",
            "address_line_4": "a4",
            "address_line_5": "a5",
            "address_line_6": "a6",
            "address_line_7": "a7",
            "address_line_8": "a8",
            "locality": "loc",
            "administrative_area": "admarea",
            "postal": "postal",
            "country": "ctry",
            "country_origin": "ctryOrg",
            "transmission_reference": "t",
            "address_opt": "addrOpt",
            "name_opt": "nameOpt",
            "email_opt": "emailOpt",
            "phone_opt": "phoneOpt",
            "opt": "opt",
            "privacy": "privacy"
        }

        # Initialize private backing fields
        for attr in self.parameter_mappings.keys():
            setattr(self, f"_{attr}", None)

        
    def _set_parameter(self, attr_name, value):
        """
        Helper method to set a private attribute and update the parameters dictionary.
        """
        setattr(self, f"_{attr_name}", value)
        self.parameters[self.parameter_mappings[attr_name]] = value

    @property
    def action(self):
        return self._action

    @action.setter
    def action(self, value):
        self._set_parameter("action", value)

    @property
    def national_id(self):
        return self._national_id

    @national_id.setter
    def national_id(self, value):
        self._set_parameter("national_id", value)

    @property
    def first_name(self):
        return self._first_name

    @first_name.setter
    def first_name(self, value):
        self._set_parameter("first_name", value)

    @property
    def last_name(self):
        return self._last_name

    @last_name.setter
    def last_name(self, value):
        self._set_parameter("last_name", value)

    @property
    def full_name(self):
        return self._full_name

    @full_name.setter
    def full_name(self, value):
        self._set_parameter("full_name", value)

    @property
    def company(self):
        return self._company

    @company.setter
    def company(self, value):
        self._set_parameter("company", value)

    @property
    def phone(self):
        return self._phone

    @phone.setter
    def phone(self, value):
        self._set_parameter("phone", value)

    @property
    def email(self):
        return self._email

    @email.setter
    def email(self, value):
        self._set_parameter("email", value)

    @property
    def dob(self):
        return self._dob

    @dob.setter
    def dob(self, value):
        self._set_parameter("dob", value)

    @property
    def account_number(self):
        return self._account_number

    @account_number.setter
    def account_number(self, value):
        self._set_parameter("account_number", value)

    @property
    def address_line_1(self):
        return self._address_line_1

    @address_line_1.setter
    def address_line_1(self, value):
        self._set_parameter("address_line_1", value)

    @property
    def address_line_2(self):
        return self._address_line_2

    @address_line_2.setter
    def address_line_2(self, value):
        self._set_parameter("address_line_2", value)

    @property
    def address_line_3(self):
        return self._address_line_3

    @address_line_3.setter
    def address_line_3(self, value):
        self._set_parameter("address_line_3", value)

    @property
    def address_line_4(self):
        return self._address_line_4

    @address_line_4.setter
    def address_line_4(self, value):
        self._set_parameter("address_line_4", value)

    @property
    def address_line_5(self):
        return self._address_line_5

    @address_line_5.setter
    def address_line_5(self, value):
        self._set_parameter("address_line_5", value)

    @property
    def address_line_6(self):
        return self._address_line_6

    @address_line_6.setter
    def address_line_6(self, value):
        self._set_parameter("address_line_6", value)

    @property
    def address_line_7(self):
        return self._address_line_7

    @address_line_7.setter
    def address_line_7(self, value):
        self._set_parameter("address_line_7", value)

    @property
    def address_line_8(self):
        return self._address_line_8

    @address_line_8.setter
    def address_line_8(self, value):
        self._set_parameter("address_line_8", value)

    @property
    def locality(self):
        return self._locality

    @locality.setter
    def locality(self, value):
        self._set_parameter("locality", value)

    @property
    def administrative_area(self):
        return self._administrative_area

    @administrative_area.setter
    def administrative_area(self, value):
        self._set_parameter("administrative_area", value)

    @property
    def postal(self):
        return self._postal

    @postal.setter
    def postal(self, value):
        self._set_parameter("postal", value)

    @property
    def country(self):
        return self._country

    @country.setter
    def country(self, value):
        self._set_parameter("country", value)

    @property
    def country_origin(self):
        return self._country_origin

    @country_origin.setter
    def country_origin(self, value):
        self._set_parameter("country_origin", value)

    @property
    def transmission_reference(self):
        return self._transmission_reference

    @transmission_reference.setter
    def transmission_reference(self, value):
        self._set_parameter("transmission_reference", value)

    @property
    def address_opt(self):
        return self._address_opt

    @address_opt.setter
    def address_opt(self, value):
        self._set_parameter("address_opt", value)

    @property
    def name_opt(self):
        return self._name_opt

    @name_opt.setter
    def name_opt(self, value):
        self._set_parameter("name_opt", value)

    @property
    def email_opt(self):
        return self._email_opt

    @email_opt.setter
    def email_opt(self, value):
        self._set_parameter("email_opt", value)

    @property
    def phone_opt(self):
        return self._phone_opt

    @phone_opt.setter
    def phone_opt(self, value):
        self._set_parameter("phone_opt", value)

    @property
    def opt(self):
        return self._opt

    @opt.setter
    def opt(self, value):
        self._set_parameter("opt", value)

    @property
    def privacy(self):
        return self._privacy

    @privacy.setter
    def privacy(self, value):
        self._set_parameter("privacy", value)


    # Setters
    def set_action(self, action):
        self._set_parameter("action", action)

    def set_national_id(self, national_id):
        self._set_parameter("national_id", national_id)

    def set_first_name(self, first_name):
        self._set_parameter("first_name", first_name)

    def set_last_name(self, last_name):
        self._set_parameter("last_name", last_name)

    def set_full_name(self, full_name):
        self._set_parameter("full_name", full_name)

    def set_company(self, company):
        self._set_parameter("company", company)

    def set_phone(self, phone):
        self._set_parameter("phone", phone)

    def set_email(self, email):
        self._set_parameter("email", email)

    def set_dob(self, dob):
        self._set_parameter("dob", dob)

    def set_account_number(self, account_number):
        self._set_parameter("account_number", account_number)

    def set_address_line_1(self, address_line_1):
        self._set_parameter("address_line_1", address_line_1)

    def set_address_line_2(self, address_line_2):
        self._set_parameter("address_line_2", address_line_2)

    def set_address_line_3(self, address_line_3):
        self._set_parameter("address_line_3", address_line_3)

    def set_address_line_4(self, address_line_4):
        self._set_parameter("address_line_4", address_line_4)

    def set_address_line_5(self, address_line_5):
        self._set_parameter("address_line_5", address_line_5)

    def set_address_line_6(self, address_line_6):
        self._set_parameter("address_line_6", address_line_6)

    def set_address_line_7(self, address_line_7):
        self._set_parameter("address_line_7", address_line_7)

    def set_address_line_8(self, address_line_8):
        self._set_parameter("address_line_8", address_line_8)

    def set_locality(self, locality):
        self._set_parameter("locality", locality)

    def set_administrative_area(self, administrative_area):
        self._set_parameter("administrative_area", administrative_area)

    def set_postal(self, postal):
        self._set_parameter("postal", postal)

    def set_country(self, country):
        self._set_parameter("country", country)

    def set_country_origin(self, country_origin):
        self._set_parameter("country_origin", country_origin)

    def set_transmission_reference(self, transmission_reference):
        self._set_parameter("transmission_reference", transmission_reference)

    def set_address_opt(self, address_opt):
        self._set_parameter("address_opt", address_opt)

    def set_name_opt(self, name_opt):
        self._set_parameter("name_opt", name_opt)

    def set_email_opt(self, email_opt):
        self._set_parameter("email_opt", email_opt)

    def set_phone_opt(self, phone_opt):
        self._set_parameter("phone_opt", phone_opt)

    def set_opt(self, opt):
        self._set_parameter("opt", opt)

    def set_privacy(self, privacy):
        self._set_parameter("privacy", privacy)

    def set_post_body(self, post_body):
        self.post_body = post_body

    def set_value(self, parameter, value):
        parameter = parameter.strip() if parameter else None
        
        if parameter in self.parameter_mappings:
            parameter_key = self.parameter_mappings[parameter]
            
            property_name = next((key for key, val in self.parameter_mappings.items() 
                                  if val.lower() == parameter_key.lower()), None)
            
            if property_name:
                if hasattr(self, property_name):
                    setattr(self, property_name, value)
        else:
            # If not in derived class, go to base class implementation
            super().set_value(parameter, value)

    # Getters
    def get_action(self):
        return self.parameters.get("act") or ""

    def get_national_id(self):
        return self.parameters.get("nat") or ""

    def get_first_name(self):
        return self.parameters.get("first") or ""

    def get_last_name(self):
        return self.parameters.get("last") or ""

    def get_full_name(self):
        return self.parameters.get("full") or ""

    def get_company(self):
        return self.parameters.get("comp") or ""

    def get_phone(self):
        return self.parameters.get("phone") or ""

    def get_email(self):
        return self.parameters.get("email") or ""

    def get_dob(self):
        return self.parameters.get("dob") or ""

    def get_account_number(self):
        return self.parameters.get("accountNumber") or ""

    def get_address_line_1(self):
        return self.parameters.get("a1") or ""

    def get_address_line_2(self):
        return self.parameters.get("a2") or ""

    def get_address_line_3(self):
        return self.parameters.get("a3") or ""

    def get_address_line_4(self):
        return self.parameters.get("a4") or ""

    def get_address_line_5(self):
        return self.parameters.get("a5") or ""

    def get_address_line_6(self):
        return self.parameters.get("a6") or ""

    def get_address_line_7(self):
        return self.parameters.get("a7") or ""

    def get_address_line_8(self):
        return self.parameters.get("a8") or ""

    def get_locality(self):
        return self.parameters.get("loc") or ""

    def get_administrative_area(self):
        return self.parameters.get("admarea") or ""

    def get_postal(self):
        return self.parameters.get("postal") or ""

    def get_country(self):
        return self.parameters.get("ctry") or ""

    def get_country_origin(self):
        return self.parameters.get("ctryOrg") or ""

    def get_transmission_reference(self):
        return self.parameters.get("t") or ""

    def get_address_opt(self):
        return self.parameters.get("addrOpt") or ""

    def get_name_opt(self):
        return self.parameters.get("nameOpt") or ""

    def get_email_opt(self):
        return self.parameters.get("emailOpt") or ""

    def get_phone_opt(self):
        return self.parameters.get("phoneOpt") or ""

    def get_opt(self):
        return self.parameters.get("opt") or ""

    def get_privacy(self):
        return self.parameters.get("privacy") or ""

    def get_post_body(self):
        return self.post_body or ""
    
    def post(self, response_type=None):
        """
        This synchronous function makes the batch request using the post body.

        Returns:
            A string or deserialized json response object.
        """

        
        response = self.send_post_request(self.post_body)
        return self.handle_response(response, response_type)

    def get_api_version(self):
        """
        Makes a synchronous getversion request and parses the response for the API version.

        Returns:
            The Cloud API version.
        """
        json_response = self.send_version_request()
        response_objectect = json.loads(json_response)
        version = response_objectect["Version"]
        return version



from .ResponseBase import ResponseBase

class PersonatorIdentityResponse(ResponseBase):
    def __init__(
            self, version="", transmission_reference="", transaction_id="", results="", peronator_identity_name=None, 
            peronator_identity_address = None, peronator_identity_email = None, peronator_identity_phone = None, peronator_identity_identity = None
    ):
        self.version = version
        self.transmission_reference = transmission_reference
        self.transaction_id = transaction_id
        self.results = results
        self.peronator_identity_name = peronator_identity_name 
        self.peronator_identity_address = peronator_identity_address 
        self.peronator_identity_email = peronator_identity_email 
        self.peronator_identity_phone = peronator_identity_phone 
        self.peronator_identity_identity = peronator_identity_identity 

    @classmethod
    def populate_from_dict(cls, data: dict):
        cls.data = data
        return cls(
            version=data.get("Version", ""),
            transmission_reference=data.get("TransmissionReference", ""),
            transaction_id=data.get("TransactionID", ""),
            results = data.get("Results", ""),
            peronator_identity_name = PersonatorIdentityName.from_dict(data.get("Name", None)),
            peronator_identity_address = PersonatorIdentityAddress.from_dict(data.get("Address", None)),
            peronator_identity_email = PersonatorIdentityEmail.from_dict(data.get("Email", None)),
            peronator_identity_phone = PersonatorIdentityPhone.from_dict(data.get("Phone", None)),
            peronator_identity_identity = PersonatorIdentityIdentity.from_dict(data.get("Identity", None))
        )
    
    # Setters
    def set_version(self, version):
        self.version = version

    def set_transmission_reference(self, transmission_reference):
        self.transmission_reference = transmission_reference

    def set_transmission_results(self, transmission_results):
        self.transmission_results = transmission_results

    def set_total_records(self, total_records):
        self.total_records = total_records

    # Getters
    def get_version(self):
        return self.version or ""

    def get_transmission_reference(self):
        return self.transmission_reference or ""

    def get_transmission_results(self):
        return self.transmission_results or ""

    def get_total_records(self):
        return self.total_records or ""
    

class PersonatorIdentityName:
    def __init__(
        self, results="", company="", name_prefix="", name_first="", name_middle="", name_last="", name_suffix="", 
        gender="", name_prefix2="", name_first2="", name_middle2="", name_last2="", name_suffix2="", gender2="",
    ):
        self.results = results
        self.company = company
        self.name_prefix = name_prefix
        self.name_first = name_first
        self.name_middle = name_middle
        self.name_last = name_last
        self.name_suffix = name_suffix
        self.gender = gender
        self.name_prefix2 = name_prefix2
        self.name_first2 = name_first2
        self.name_middle2 = name_middle2
        self.name_last2 = name_last2
        self.name_suffix2 = name_suffix2
        self.gender2 = gender2

    @classmethod
    def from_dict(cls, data: dict):
        cls.data = data
        print(data)
        return cls(
            results=data.get("Results", ""),
            company=data.get("Company", ""),
            name_prefix=data.get("NamePrefix", ""),
            name_first=data.get("NameFirst", ""),
            name_middle=data.get("NameMiddle", ""),
            name_last=data.get("NameLast", ""),
            name_suffix=data.get("NameSuffix", ""),
            gender=data.get("Gender", ""),
            name_prefix2=data.get("NamePrefix2", ""),
            name_first2=data.get("NameFirst2", ""),
            name_middle2=data.get("NameMiddle2", ""),
            name_last2=data.get("NameLast2", ""),
            name_suffix2=data.get("NameSuffix2", ""),
            gender2=data.get("Gender2", ""),
        )
    
    # Setters
    def set_results(self, results):
        self.results = results

    def set_company(self, company):
        self.company = company

    def set_name_prefix(self, name_prefix):
        self.name_prefix = name_prefix

    def set_name_first(self, name_first):
        self.name_first = name_first

    def set_name_middle(self, name_middle):
        self.name_middle = name_middle

    def set_name_last(self, name_last):
        self.name_last = name_last

    def set_name_suffix(self, name_suffix):
        self.name_suffix = name_suffix

    def set_gender(self, gender):
        self.gender = gender

    def set_name_prefix2(self, name_prefix2):
        self.name_prefix2 = name_prefix2

    def set_name_first2(self, name_first2):
        self.name_first2 = name_first2

    def set_name_middle2(self, name_middle2):
        self.name_middle2 = name_middle2

    def set_name_last2(self, name_last2):
        self.name_last2 = name_last2

    def set_name_suffix2(self, name_suffix2):
        self.name_suffix2 = name_suffix2

    def set_gender2(self, gender2):
        self.gender2 = gender2

    # Getters
    def get_results(self):
        return self.results or ""

    def get_company(self):
        return self.company or ""

    def get_name_prefix(self):
        return self.name_prefix or ""

    def get_name_first(self):
        return self.name_first or ""

    def get_name_middle(self):
        return self.name_middle or ""

    def get_name_last(self):
        return self.name_last or ""

    def get_name_suffix(self):
        return self.name_suffix or ""

    def get_gender(self):
        return self.gender or ""

    def get_name_prefix2(self):
        return self.name_prefix2 or ""

    def get_name_first2(self):
        return self.name_first2 or ""

    def get_name_middle2(self):
        return self.name_middle2 or ""

    def get_name_last2(self):
        return self.name_last2 or ""

    def get_name_suffix2(self):
        return self.name_suffix2 or ""

    def get_gender2(self):
        return self.gender2 or ""


class PersonatorIdentityAddress:
    def __init__(
        self, results="", formatted_address="", organization="", address_line_1="", address_line_2="", address_line_3="", address_line_4="", 
        address_line_5="", address_line_6="", address_line_7="", address_line_8="", sub_premises="", double_dependent_locality="", 
        dependent_locality="", locality="", sub_administrative_area="", administrative_area="", postal_code="", address_type="", address_key="", 
        sub_national_area="", country_name="", country_code="", country_iso3="", country_number="", country_subdivision_code="",
        thoroughfare="", thoroughfare_pre_direction="", thoroughfare_leading_type="", thoroughfare_name="", thoroughfare_trailing_type="", 
        thoroughfare_post_direction="", dependent_thoroughfare="", dependent_thoroughfare_pre_direction="", dependent_thoroughfare_leading_type="",
        dependent_thoroughfare_name="", dependent_thoroughfare_trailing_type="", dependent_thoroughfare_post_direction="", building="", 
        premises_type="", premises_number="", sub_premises_type="", sub_premises_number="", post_box="", latitude="", longitude="",
    ):
        self.results = results
        self.formatted_address = formatted_address
        self.organization = organization
        self.address_line_1 = address_line_1
        self.address_line_2 = address_line_2
        self.address_line_3 = address_line_3
        self.address_line_4 = address_line_4
        self.address_line_5 = address_line_5
        self.address_line_6 = address_line_6
        self.address_line_7 = address_line_7
        self.address_line_8 = address_line_8
        self.sub_premises = sub_premises
        self.double_dependent_locality = double_dependent_locality
        self.dependent_locality = dependent_locality
        self.locality = locality
        self.sub_administrative_area = sub_administrative_area
        self.administrative_area = administrative_area
        self.postal_code = postal_code
        self.address_type = address_type
        self.address_key = address_key
        self.sub_national_area = sub_national_area
        self.country_name = country_name
        self.country_code = country_code
        self.country_iso3 = country_iso3
        self.country_number = country_number
        self.country_subdivision_code = country_subdivision_code
        self.thoroughfare = thoroughfare
        self.thoroughfare_pre_direction = thoroughfare_pre_direction
        self.thoroughfare_leading_type = thoroughfare_leading_type
        self.thoroughfare_name = thoroughfare_name
        self.thoroughfare_trailing_type = thoroughfare_trailing_type
        self.thoroughfare_post_direction = thoroughfare_post_direction
        self.dependent_thoroughfare = dependent_thoroughfare
        self.dependent_thoroughfare_pre_direction = dependent_thoroughfare_pre_direction
        self.dependent_thoroughfare_leading_type = dependent_thoroughfare_leading_type
        self.dependent_thoroughfare_name = dependent_thoroughfare_name
        self.dependent_thoroughfare_trailing_type = dependent_thoroughfare_trailing_type
        self.dependent_thoroughfare_post_direction = dependent_thoroughfare_post_direction
        self.building = building
        self.premises_type = premises_type
        self.premises_number = premises_number
        self.sub_premises_type = sub_premises_type
        self.sub_premises_number = sub_premises_number
        self.post_box = post_box
        self.latitude = latitude
        self.longitude = longitude

    @classmethod
    def from_dict(cls, data):
        cls.data = data
        return cls(
            results=data.get("Results", ""),
            formatted_address=data.get("FormattedAddress", ""),
            organization=data.get("Organization", ""),
            address_line_1=data.get("AddressLine1", ""),
            address_line_2=data.get("AddressLine2", ""),
            address_line_3=data.get("AddressLine3", ""),
            address_line_4=data.get("AddressLine4", ""),
            address_line_5=data.get("AddressLine5", ""),
            address_line_6=data.get("AddressLine6", ""),
            address_line_7=data.get("AddressLine7", ""),
            address_line_8=data.get("AddressLine8", ""),
            sub_premises=data.get("SubPremises", ""),
            double_dependent_locality=data.get("DoubleDependentLocality", ""),
            dependent_locality=data.get("DependentLocality", ""),
            locality=data.get("Locality", ""),
            sub_administrative_area=data.get("SubAdministrativeArea", ""),
            administrative_area=data.get("AdministrativeArea", ""),
            postal_code=data.get("PostalCode", ""),
            address_type=data.get("AddressType", ""),
            address_key=data.get("AddressKey", ""),
            sub_national_area=data.get("SubNationalArea", ""),
            country_name=data.get("CountryName", ""),
            country_code=data.get("CountryCode", ""),
            country_iso3=data.get("CountryISO3", ""),
            country_number=data.get("CountryNumber", ""),
            country_subdivision_code=data.get("CountrySubdivisionCode", ""),
            thoroughfare=data.get("Thoroughfare", ""),
            thoroughfare_pre_direction=data.get("ThoroughfarePreDirection", ""),
            thoroughfare_leading_type=data.get("ThoroughfareLeadingType", ""),
            thoroughfare_name=data.get("ThoroughfareName", ""),
            thoroughfare_trailing_type=data.get("ThoroughfareTrailingType", ""),
            thoroughfare_post_direction=data.get("ThoroughfarePostDirection", ""),
            dependent_thoroughfare=data.get("DependentThoroughfare", ""),
            dependent_thoroughfare_pre_direction=data.get("DependentThoroughfarePreDirection", ""),
            dependent_thoroughfare_leading_type=data.get("DependentThoroughfareLeadingType", ""),
            dependent_thoroughfare_name=data.get("DependentThoroughfareName", ""),
            dependent_thoroughfare_trailing_type=data.get("DependentThoroughfareTrailingType", ""),
            dependent_thoroughfare_post_direction=data.get("DependentThoroughfarePostDirection", ""),
            building=data.get("Building", ""),
            premises_type=data.get("PremisesType", ""),
            premises_number=data.get("PremisesNumber", ""),
            sub_premises_type=data.get("SubPremisesType", ""),
            sub_premises_number=data.get("SubPremisesNumber", ""),
            post_box=data.get("PostBox", ""),
            latitude=data.get("Latitude", ""),
            longitude=data.get("Longitude", ""),
        )
    
    # Setters
    def set_results(self, results):
        self.results = results

    def set_formatted_address(self, formatted_address):
        self.formatted_address = formatted_address

    def set_organization(self, organization):
        self.organization = organization

    def set_address_line_1(self, address_line_1):
        self.address_line_1 = address_line_1

    def set_address_line_2(self, address_line_2):
        self.address_line_2 = address_line_2

    def set_address_line_3(self, address_line_3):
        self.address_line_3 = address_line_3

    def set_address_line_4(self, address_line_4):
        self.address_line_4 = address_line_4

    def set_address_line_5(self, address_line_5):
        self.address_line_5 = address_line_5

    def set_address_line_6(self, address_line_6):
        self.address_line_6 = address_line_6

    def set_address_line_7(self, address_line_7):
        self.address_line_7 = address_line_7

    def set_address_line_8(self, address_line_8):
        self.address_line_8 = address_line_8

    def set_sub_premises(self, sub_premises):
        self.sub_premises = sub_premises

    def set_double_dependent_locality(self, double_dependent_locality):
        self.double_dependent_locality = double_dependent_locality

    def set_dependent_locality(self, dependent_locality):
        self.dependent_locality = dependent_locality

    def set_locality(self, locality):
        self.locality = locality

    def set_sub_administrative_area(self, sub_administrative_area):
        self.sub_administrative_area = sub_administrative_area

    def set_administrative_area(self, administrative_area):
        self.administrative_area = administrative_area

    def set_postal_code(self, postal_code):
        self.postal_code = postal_code

    def set_address_type(self, address_type):
        self.address_type = address_type

    def set_address_key(self, address_key):
        self.address_key = address_key

    def set_sub_national_area(self, sub_national_area):
        self.sub_national_area = sub_national_area

    def set_country_name(self, country_name):
        self.country_name = country_name

    def set_country_code(self, country_code):
        self.country_code = country_code

    def set_country_iso3(self, country_iso3):
        self.country_iso3 = country_iso3

    def set_country_number(self, country_number):
        self.country_number = country_number

    def set_country_subdivision_code(self, country_subdivision_code):
        self.country_subdivision_code = country_subdivision_code

    def set_thoroughfare(self, thoroughfare):
        self.thoroughfare = thoroughfare

    def set_thoroughfare_pre_direction(self, thoroughfare_pre_direction):
        self.thoroughfare_pre_direction = thoroughfare_pre_direction

    def set_thoroughfare_leading_type(self, thoroughfare_leading_type):
        self.thoroughfare_leading_type = thoroughfare_leading_type

    def set_thoroughfare_name(self, thoroughfare_name):
        self.thoroughfare_name = thoroughfare_name

    def set_thoroughfare_trailing_type(self, thoroughfare_trailing_type):
        self.thoroughfare_trailing_type = thoroughfare_trailing_type

    def set_thoroughfare_post_direction(self, thoroughfare_post_direction):
        self.thoroughfare_post_direction = thoroughfare_post_direction

    def set_dependent_thoroughfare(self, dependent_thoroughfare):
        self.dependent_thoroughfare = dependent_thoroughfare

    def set_dependent_thoroughfare_pre_direction(self, dependent_thoroughfare_pre_direction):
        self.dependent_thoroughfare_pre_direction = dependent_thoroughfare_pre_direction

    def set_dependent_thoroughfare_leading_type(self, dependent_thoroughfare_leading_type):
        self.dependent_thoroughfare_leading_type = dependent_thoroughfare_leading_type

    def set_dependent_thoroughfare_name(self, dependent_thoroughfare_name):
        self.dependent_thoroughfare_name = dependent_thoroughfare_name

    def set_dependent_thoroughfare_trailing_type(self, dependent_thoroughfare_trailing_type):
        self.dependent_thoroughfare_trailing_type = dependent_thoroughfare_trailing_type

    def set_dependent_thoroughfare_post_direction(self, dependent_thoroughfare_post_direction):
        self.dependent_thoroughfare_post_direction = dependent_thoroughfare_post_direction

    def set_building(self, building):
        self.building = building

    def set_premises_type(self, premises_type):
        self.premises_type = premises_type

    def set_premises_number(self, premises_number):
        self.premises_number = premises_number

    def set_sub_premises_type(self, sub_premises_type):
        self.sub_premises_type = sub_premises_type

    def set_sub_premises_number(self, sub_premises_number):
        self.sub_premises_number = sub_premises_number

    def set_post_box(self, post_box):
        self.post_box = post_box

    def set_latitude(self, latitude):
        self.latitude = latitude

    def set_longitude(self, longitude):
        self.longitude = longitude

    # Getters
    def get_results(self):
        return self.results or ""

    def get_formatted_address(self):
        return self.formatted_address or ""

    def get_organization(self):
        return self.organization or ""

    def get_address_line_1(self):
        return self.address_line_1 or ""

    def get_address_line_2(self):
        return self.address_line_2 or ""

    def get_address_line_3(self):
        return self.address_line_3 or ""

    def get_address_line_4(self):
        return self.address_line_4 or ""

    def get_address_line_5(self):
        return self.address_line_5 or ""

    def get_address_line_6(self):
        return self.address_line_6 or ""

    def get_address_line_7(self):
        return self.address_line_7 or ""

    def get_address_line_8(self):
        return self.address_line_8 or ""

    def get_sub_premises(self):
        return self.sub_premises or ""

    def get_double_dependent_locality(self):
        return self.double_dependent_locality or ""

    def get_dependent_locality(self):
        return self.dependent_locality or ""

    def get_locality(self):
        return self.locality or ""

    def get_sub_administrative_area(self):
        return self.sub_administrative_area or ""

    def get_administrative_area(self):
        return self.administrative_area or ""

    def get_postal_code(self):
        return self.postal_code or ""

    def get_address_type(self):
        return self.address_type or ""

    def get_address_key(self):
        return self.address_key or ""

    def get_sub_national_area(self):
        return self.sub_national_area or ""

    def get_country_name(self):
        return self.country_name or ""

    def get_country_code(self):
        return self.country_code or ""

    def get_country_iso3(self):
        return self.country_iso3 or ""

    def get_country_number(self):
        return self.country_number or ""

    def get_country_subdivision_code(self):
        return self.country_subdivision_code or ""

    def get_thoroughfare(self):
        return self.thoroughfare or ""

    def get_thoroughfare_pre_direction(self):
        return self.thoroughfare_pre_direction or ""

    def get_thoroughfare_leading_type(self):
        return self.thoroughfare_leading_type or ""

    def get_thoroughfare_name(self):
        return self.thoroughfare_name or ""

    def get_thoroughfare_trailing_type(self):
        return self.thoroughfare_trailing_type or ""

    def get_thoroughfare_post_direction(self):
        return self.thoroughfare_post_direction or ""

    def get_dependent_thoroughfare(self):
        return self.dependent_thoroughfare or ""

    def get_dependent_thoroughfare_pre_direction(self):
        return self.dependent_thoroughfare_pre_direction or ""

    def get_dependent_thoroughfare_leading_type(self):
        return self.dependent_thoroughfare_leading_type or ""

    def get_dependent_thoroughfare_name(self):
        return self.dependent_thoroughfare_name or ""

    def get_dependent_thoroughfare_trailing_type(self):
        return self.dependent_thoroughfare_trailing_type or ""

    def get_dependent_thoroughfare_post_direction(self):
        return self.dependent_thoroughfare_post_direction or ""

    def get_building(self):
        return self.building or ""

    def get_premises_type(self):
        return self.premises_type or ""

    def get_premises_number(self):
        return self.premises_number or ""

    def get_sub_premises_type(self):
        return self.sub_premises_type or ""

    def get_sub_premises_number(self):
        return self.sub_premises_number or ""

    def get_post_box(self):
        return self.post_box or ""

    def get_latitude(self):
        return self.latitude or ""

    def get_longitude(self):
        return self.longitude or ""
    
class PersonatorIdentityEmail(ResponseBase):
    def __init__(
        self, deliverability_confidence_score="", results="", email_address="", mailbox_name="", domain_name="", 
        top_level_domain="", top_level_domain_name="", date_checked="", email_age_estimated="", domain_age_estimated="", 
        domain_expiration_date="", domain_created_date="", domain_updated_date="", domain_email="", domain_organization="", 
        domain_address1="", domain_locality="", domain_administrative_area="", domain_postal_code="", domain_country="", 
        domain_availability="", domain_country_code="", domain_private_proxy="", privacy_flag="", mx_server="", breach_count=""
    ):
        self.deliverability_confidence_score = deliverability_confidence_score
        self.results = results
        self.email_address = email_address
        self.mailbox_name = mailbox_name
        self.domain_name = domain_name
        self.top_level_domain = top_level_domain
        self.top_level_domain_name = top_level_domain_name
        self.date_checked = date_checked
        self.email_age_estimated = email_age_estimated
        self.domain_age_estimated = domain_age_estimated
        self.domain_expiration_date = domain_expiration_date
        self.domain_created_date = domain_created_date
        self.domain_updated_date = domain_updated_date
        self.domain_email = domain_email
        self.domain_organization = domain_organization
        self.domain_address1 = domain_address1
        self.domain_locality = domain_locality
        self.domain_administrative_area = domain_administrative_area
        self.domain_postal_code = domain_postal_code
        self.domain_country = domain_country
        self.domain_availability = domain_availability
        self.domain_country_code = domain_country_code
        self.domain_private_proxy = domain_private_proxy
        self.privacy_flag = privacy_flag
        self.mx_server = mx_server
        self.breach_count = breach_count

    @classmethod
    def from_dict(cls, data):
        cls.data = data
        return cls(
            deliverability_confidence_score=data.get("DeliverabilityConfidenceScore", ""),
            results=data.get("Results", ""),
            email_address=data.get("EmailAddress", ""),
            mailbox_name=data.get("MailboxName", ""),
            domain_name=data.get("DomainName", ""),
            top_level_domain=data.get("TopLevelDomain", ""),
            top_level_domain_name=data.get("TopLevelDomainName", ""),
            date_checked=data.get("DateChecked", ""),
            email_age_estimated=data.get("EmailAgeEstimated", ""),
            domain_age_estimated=data.get("DomainAgeEstimated", ""),
            domain_expiration_date=data.get("DomainExpirationDate", ""),
            domain_created_date=data.get("DomainCreatedDate", ""),
            domain_updated_date=data.get("DomainUpdatedDate", ""),
            domain_email=data.get("DomainEmail", ""),
            domain_organization=data.get("DomainOrganization", ""),
            domain_address1=data.get("DomainAddress1", ""),
            domain_locality=data.get("DomainLocality", ""),
            domain_administrative_area=data.get("DomainAdministrativeArea", ""),
            domain_postal_code=data.get("DomainPostalCode", ""),
            domain_country=data.get("DomainCountry", ""),
            domain_availability=data.get("DomainAvailability", ""),
            domain_country_code=data.get("DomainCountryCode", ""),
            domain_private_proxy=data.get("DomainPrivateProxy", ""),
            privacy_flag=data.get("PrivacyFlag", ""),
            mx_server=data.get("MXServer", ""),
            breach_count=data.get("BreachCount", "")
        )
    
    # Setters
    def set_deliverability_confidence_score(self, deliverability_confidence_score):
        self.deliverability_confidence_score = deliverability_confidence_score

    def set_results(self, results):
        self.results = results

    def set_email_address(self, email_address):
        self.email_address = email_address

    def set_mailbox_name(self, mailbox_name):
        self.mailbox_name = mailbox_name

    def set_domain_name(self, domain_name):
        self.domain_name = domain_name

    def set_top_level_domain(self, top_level_domain):
        self.top_level_domain = top_level_domain

    def set_top_level_domain_name(self, top_level_domain_name):
        self.top_level_domain_name = top_level_domain_name

    def set_date_checked(self, date_checked):
        self.date_checked = date_checked

    def set_email_age_estimated(self, email_age_estimated):
        self.email_age_estimated = email_age_estimated

    def set_domain_age_estimated(self, domain_age_estimated):
        self.domain_age_estimated = domain_age_estimated

    def set_domain_expiration_date(self, domain_expiration_date):
        self.domain_expiration_date = domain_expiration_date

    def set_domain_created_date(self, domain_created_date):
        self.domain_created_date = domain_created_date

    def set_domain_updated_date(self, domain_updated_date):
        self.domain_updated_date = domain_updated_date

    def set_domain_email(self, domain_email):
        self.domain_email = domain_email

    def set_domain_organization(self, domain_organization):
        self.domain_organization = domain_organization

    def set_domain_address1(self, domain_address1):
        self.domain_address1 = domain_address1

    def set_domain_locality(self, domain_locality):
        self.domain_locality = domain_locality

    def set_domain_administrative_area(self, domain_administrative_area):
        self.domain_administrative_area = domain_administrative_area

    def set_domain_postal_code(self, domain_postal_code):
        self.domain_postal_code = domain_postal_code

    def set_domain_country(self, domain_country):
        self.domain_country = domain_country

    def set_domain_availability(self, domain_availability):
        self.domain_availability = domain_availability

    def set_domain_country_code(self, domain_country_code):
        self.domain_country_code = domain_country_code

    def set_domain_private_proxy(self, domain_private_proxy):
        self.domain_private_proxy = domain_private_proxy

    def set_privacy_flag(self, privacy_flag):
        self.privacy_flag = privacy_flag

    def set_mx_server(self, mx_server):
        self.mx_server = mx_server

    def set_breach_count(self, breach_count):
        self.breach_count = breach_count

    # Getters
    def get_deliverability_confidence_score(self):
        return self.deliverability_confidence_score or ""

    def get_results(self):
        return self.results or ""

    def get_email_address(self):
        return self.email_address or ""

    def get_mailbox_name(self):
        return self.mailbox_name or ""

    def get_domain_name(self):
        return self.domain_name or ""

    def get_top_level_domain(self):
        return self.top_level_domain or ""

    def get_top_level_domain_name(self):
        return self.top_level_domain_name or ""

    def get_date_checked(self):
        return self.date_checked or ""

    def get_email_age_estimated(self):
        return self.email_age_estimated or ""

    def get_domain_age_estimated(self):
        return self.domain_age_estimated or ""

    def get_domain_expiration_date(self):
        return self.domain_expiration_date or ""

    def get_domain_created_date(self):
        return self.domain_created_date or ""

    def get_domain_updated_date(self):
        return self.domain_updated_date or ""

    def get_domain_email(self):
        return self.domain_email or ""

    def get_domain_organization(self):
        return self.domain_organization or ""

    def get_domain_address1(self):
        return self.domain_address1 or ""

    def get_domain_locality(self):
        return self.domain_locality or ""

    def get_domain_administrative_area(self):
        return self.domain_administrative_area or ""

    def get_domain_postal_code(self):
        return self.domain_postal_code or ""

    def get_domain_country(self):
        return self.domain_country or ""

    def get_domain_availability(self):
        return self.domain_availability or ""

    def get_domain_country_code(self):
        return self.domain_country_code or ""

    def get_domain_private_proxy(self):
        return self.domain_private_proxy or ""

    def get_privacy_flag(self):
        return self.privacy_flag or ""

    def get_mx_server(self):
        return self.mx_server or ""

    def get_breach_count(self):
        return self.breach_count or ""

class PersonatorIdentityPhone(ResponseBase):
    def __init__(
        self, results="", phone_number="", administrative_area="", country_abbreviation="", country_name="", carrier="", 
        caller_id="", international_phone_number="", language="", latitude="", locality="", longitude="", phone_international_prefix="", 
        phone_country_dialing_code="", phone_nation_prefix="", phone_national_destination_code="", phone_subscriber_number="", utc="", 
        postal_code="", suggestions="", dst="", time_zone_code="", time_zone_name=""
    ):
        self.results = results
        self.phone_number = phone_number
        self.administrative_area = administrative_area
        self.country_abbreviation = country_abbreviation
        self.country_name = country_name
        self.carrier = carrier
        self.caller_id = caller_id
        self.international_phone_number = international_phone_number
        self.language = language
        self.latitude = latitude
        self.locality = locality
        self.longitude = longitude
        self.phone_international_prefix = phone_international_prefix
        self.phone_country_dialing_code = phone_country_dialing_code
        self.phone_nation_prefix = phone_nation_prefix
        self.phone_national_destination_code = phone_national_destination_code
        self.phone_subscriber_number = phone_subscriber_number
        self.utc = utc
        self.postal_code = postal_code
        self.suggestions = suggestions
        self.dst = dst
        self.time_zone_code = time_zone_code
        self.time_zone_name = time_zone_name

    @classmethod
    def from_dict(cls, data):
        cls.data = data
        return cls(
            results=data.get("Results", ""),
            phone_number=data.get("PhoneNumber", ""),
            administrative_area=data.get("AdministrativeArea", ""),
            country_abbreviation=data.get("CountryAbbreviation", ""),
            country_name=data.get("CountryName", ""),
            carrier=data.get("Carrier", ""),
            caller_id=data.get("CallerID", ""),
            international_phone_number=data.get("InternationalPhoneNumber", ""),
            language=data.get("Language", ""),
            latitude=data.get("Latitude", ""),
            locality=data.get("Locality", ""),
            longitude=data.get("Longitude", ""),
            phone_international_prefix=data.get("PhoneInternationalPrefix", ""),
            phone_country_dialing_code=data.get("PhoneCountryDialingCode", ""),
            phone_nation_prefix=data.get("PhoneNationPrefix", ""),
            phone_national_destination_code=data.get("PhoneNationalDestinationCode", ""),
            phone_subscriber_number=data.get("PhoneSubscriberNumber", ""),
            utc=data.get("UTC", ""),
            postal_code=data.get("PostalCode", ""),
            suggestions=data.get("Suggestions", ""),
            dst=data.get("DST", ""),
            time_zone_code=data.get("TimeZoneCode", ""),
            time_zone_name=data.get("TimeZoneName", "")
        )
    
    # Setters
    def set_results(self, results):
        self.results = results

    def set_phone_number(self, phone_number):
        self.phone_number = phone_number

    def set_administrative_area(self, administrative_area):
        self.administrative_area = administrative_area

    def set_country_abbreviation(self, country_abbreviation):
        self.country_abbreviation = country_abbreviation

    def set_country_name(self, country_name):
        self.country_name = country_name

    def set_carrier(self, carrier):
        self.carrier = carrier

    def set_caller_id(self, caller_id):
        self.caller_id = caller_id

    def set_international_phone_number(self, international_phone_number):
        self.international_phone_number = international_phone_number

    def set_language(self, language):
        self.language = language

    def set_latitude(self, latitude):
        self.latitude = latitude

    def set_locality(self, locality):
        self.locality = locality

    def set_longitude(self, longitude):
        self.longitude = longitude

    def set_phone_international_prefix(self, phone_international_prefix):
        self.phone_international_prefix = phone_international_prefix

    def set_phone_country_dialing_code(self, phone_country_dialing_code):
        self.phone_country_dialing_code = phone_country_dialing_code

    def set_phone_nation_prefix(self, phone_nation_prefix):
        self.phone_nation_prefix = phone_nation_prefix

    def set_phone_national_destination_code(self, phone_national_destination_code):
        self.phone_national_destination_code = phone_national_destination_code

    def set_phone_subscriber_number(self, phone_subscriber_number):
        self.phone_subscriber_number = phone_subscriber_number

    def set_utc(self, utc):
        self.utc = utc

    def set_postal_code(self, postal_code):
        self.postal_code = postal_code

    def set_suggestions(self, suggestions):
        self.suggestions = suggestions

    def set_dst(self, dst):
        self.dst = dst

    def set_time_zone_code(self, time_zone_code):
        self.time_zone_code = time_zone_code

    def set_time_zone_name(self, time_zone_name):
        self.time_zone_name = time_zone_name

    # Getters
    def get_results(self):
        return self.results or ""

    def get_phone_number(self):
        return self.phone_number or ""

    def get_administrative_area(self):
        return self.administrative_area or ""

    def get_country_abbreviation(self):
        return self.country_abbreviation or ""

    def get_country_name(self):
        return self.country_name or ""

    def get_carrier(self):
        return self.carrier or ""

    def get_caller_id(self):
        return self.caller_id or ""

    def get_international_phone_number(self):
        return self.international_phone_number or ""

    def get_language(self):
        return self.language or ""

    def get_latitude(self):
        return self.latitude or ""

    def get_locality(self):
        return self.locality or ""

    def get_longitude(self):
        return self.longitude or ""

    def get_phone_international_prefix(self):
        return self.phone_international_prefix or ""

    def get_phone_country_dialing_code(self):
        return self.phone_country_dialing_code or ""

    def get_phone_nation_prefix(self):
        return self.phone_nation_prefix or ""

    def get_phone_national_destination_code(self):
        return self.phone_national_destination_code or ""

    def get_phone_subscriber_number(self):
        return self.phone_subscriber_number or ""

    def get_utc(self):
        return self.utc or ""

    def get_postal_code(self):
        return self.postal_code or ""

    def get_suggestions(self):
        return self.suggestions or ""

    def get_dst(self):
        return self.dst or ""

    def get_time_zone_code(self):
        return self.time_zone_code or ""

    def get_time_zone_name(self):
        return self.time_zone_name or ""


class PersonatorIdentityIdentity(ResponseBase):
    def __init__(self, results="", confidence="", datasources=None, watchlists=None, 
                 watchlist_persons="", national_id="", date_of_birth="", account_number=None, 
                 business_information=None
    ):
        self.results = results
        self.confidence = confidence
        self.datasources = datasources if datasources is not None else []
        self.watchlists = watchlists
        self.watchlist_persons = watchlist_persons
        self.national_id = national_id
        self.date_of_birth = date_of_birth
        self.account_number = account_number
        self.business_information = business_information

    @classmethod
    def from_dict(cls, data):
        cls.data = data
        return cls(
            results=data.get("Results", ""),
            confidence=data.get("Confidence", ""),
            datasources=data.get("Datasources", []),
            watchlists=IdentityWatchlists.from_dict(data["Watchlists"]) if data.get("Watchlists") else None,
            watchlist_persons=data.get("WatchlistPersons", ""),
            national_id=data.get("NationalID", ""),
            date_of_birth=data.get("DateOfBirth", ""),
            account_number=IdentityAccountNumber.from_dict(data["AccountNumber"]) if data.get("AccountNumber") else None,
            business_information=IdentityBusinessInformation.from_dict(data["BusinessInformation"]) if data.get("BusinessInformation") else None
        )

    # Setters
    def set_results(self, results):
        self.results = results

    def set_confidence(self, confidence):
        self.confidence = confidence

    def set_datasources(self, datasources):
        self.datasources = datasources

    def set_watchlists(self, watchlists):
        self.watchlists = watchlists

    def set_watchlist_persons(self, watchlist_persons):
        self.watchlist_persons = watchlist_persons

    def set_national_id(self, national_id):
        self.national_id = national_id

    def set_date_of_birth(self, date_of_birth):
        self.date_of_birth = date_of_birth

    def set_account_number(self, account_number):
        self.account_number = account_number

    def set_business_information(self, business_information):
        self.business_information = business_information
    
    # Getters
    def get_results(self):
        return self.results or ""

    def get_confidence(self):
        return self.confidence or ""

    def get_datasources(self):
        return self.datasources or []

    def get_watchlists(self):
        return self.watchlists or None

    def get_watchlist_persons(self):
        return self.watchlist_persons or ""

    def get_national_id(self):
        return self.national_id or ""

    def get_date_of_birth(self):
        return self.date_of_birth or ""

    def get_account_number(self):
        return self.account_number or None

    def get_business_information(self):
        return self.business_information or None

class IdentityWatchlists(ResponseBase):
    def __init__(self, hit="", sources=None, articles=None, report_link=""):
        self.hit = hit
        self.sources = sources if sources is not None else []
        self.articles = articles if articles is not None else []
        self.report_link = report_link

    @classmethod
    def from_dict(cls, data):
        cls.data = data
        return cls(
            hit=data.get("Hit", ""),
            sources=data.get("Sources", []),
            articles=data.get("Articles", []),
            report_link=data.get("ReportLink", "")
        )

    # Setters
    def set_hit(self, hit):
        self.hit = hit

    def set_report_link(self, report_link):
        self.report_link = report_link

    # Getters
    def get_hit(self):
        return self.hit or ""

    def get_report_link(self):
        return self.report_link or ""
    

class IdentityAccountNumber(ResponseBase):
    def __init__(self, validated=""):
        self.validated = validated

    @classmethod
    def from_dict(cls, data):
        cls.data = data
        return cls(
            validated=data.get("Validated", "")
        )

    # Setters
    def set_validated(self, validated):
        self.validated = validated

    # Getters
    def get_validated(self):
        return self.validated or ""

class IdentityBusinessInformation(ResponseBase):
    def __init__(self, name="", id="", status="", type="", registration_date=""):
        self.name = name
        self.id = id
        self.status = status
        self.type = type
        self.registration_date = registration_date

    @classmethod
    def from_dict(cls, data):
        cls.data = data
        return cls(
            name=data.get("Name", ""),
            id=data.get("ID", ""),
            status=data.get("Status", ""),
            type=data.get("Type", ""),
            registration_date=data.get("RegistrationDate", "")
        )

    # Setters
    def set_name(self, name):
        self.name = name

    def set_id(self, id):
        self.id = id

    def set_status(self, status):
        self.status = status

    def set_type(self, type):
        self.type = type

    def set_registration_date(self, registration_date):
        self.registration_date = registration_date

    # Getters
    def get_name(self):
        return self.name or ""

    def get_id(self):
        return self.id or ""

    def get_status(self):
        return self.status or ""

    def get_type(self):
        return self.type or ""

    def get_registration_date(self):
        return self.registration_date or ""




from .ResponseBase import ResponseBase

class PersonatorSearchResponse(ResponseBase):

    def __init__(self, version="", transmission_reference="", transmission_results="", total_pages="", total_records="", records=None):
        self.version = version
        self.transmission_reference = transmission_reference
        self.transmission_results = transmission_results
        self.total_pages = total_pages
        self.total_records = total_records
        self.records = records if records is not None else []

    @classmethod
    def populate_from_dict(cls, data: dict):
        cls.data = data
        records = [PersonatorSearchRecord.from_dict(record) for record in data.get("Records", [])]
        return cls(
            version=data.get("Version", ""),
            transmission_reference=data.get("TransmissionReference", ""),
            transmission_results=data.get("TransmissionResults", ""),
            total_pages=data.get("TotalPages", ""),
            total_records=data.get("TotalRecords", ""),
            records=records
        )

    # Setters
    def set_version(self, version):
        self.version = version

    def set_transmission_reference(self, transmission_reference):
        self.transmission_reference = transmission_reference

    def set_transmission_results(self, transmission_results):
        self.transmission_results = transmission_results

    def set_total_pages(self, total_pages):
        self.total_pages = total_pages

    def set_total_records(self, total_records):
        self.total_records = total_records

    # Getters
    def get_version(self):
        return self.version or ""

    def get_transmission_reference(self):
        return self.transmission_reference or ""

    def get_transmission_results(self):
        return self.transmission_results or ""
    
    def get_total_pages(self):
        return self.total_pages or ""

    def get_total_records(self):
        return self.total_records or ""
    

class PersonatorSearchRecord:
    def __init__(
        self, record_id="", results="", full_name="", first_name="", last_name="", date_of_birth="", date_of_death="", \
            melissa_identity_key="", current_address=None, previous_addresses=None, phone_records=None, email_records=None,
    ):
        self.record_id = record_id
        self.results = results
        self.full_name = full_name
        self.first_name = first_name
        self.last_name = last_name
        self.date_of_birth = date_of_birth
        self.date_of_death = date_of_death
        self.melissa_identity_key = melissa_identity_key
        self.current_address = current_address
        self.previous_addresses = previous_addresses or []
        self.phone_records = phone_records or []
        self.email_records = email_records or []

    @classmethod
    def from_dict(cls, data):
        cls.data = data

        return cls(
            record_id=data.get("RecordID", ""),
            results=data.get("Results", ""),
            full_name=data.get("FullName", ""),
            first_name=data.get("FirstName", ""),
            last_name=data.get("LastName", ""),
            date_of_birth=data.get("DateOfBirth", ""),
            date_of_death=data.get("DateOfDeath", ""),
            melissa_identity_key=data.get("MelissaIdentityKey", ""),
            current_address = PersonatorSearchAddress.from_dict(data.get("CurrentAddress", None)),
            previous_addresses = [PersonatorSearchAddress.from_dict(addr) for addr in data.get("PreviousAddresses", [])],
            phone_records = [PersonatorSearchPhone.from_dict(records) for records in data.get("PhoneRecords", [])],
            email_records = data.get("EmailRecords", [])
        )
    
     # Setters
    def set_record_id(self, record_id):
        self.record_id = record_id

    def set_results(self, results):
        self.results = results

    def set_full_name(self, full_name):
        self.full_name = full_name

    def set_first_name(self, first_name):
        self.first_name = first_name

    def set_last_name(self, last_name):
        self.last_name = last_name

    def set_date_of_birth(self, date_of_birth):
        self.date_of_birth = date_of_birth

    def set_date_of_death(self, date_of_death):
        self.date_of_death = date_of_death

    def set_melissa_identity_key(self, melissa_identity_key):
        self.melissa_identity_key = melissa_identity_key

    # Getters
    def get_record_id(self):
        return self.record_id or ""

    def get_results(self):
        return self.results or ""

    def get_full_name(self):
        return self.full_name or ""

    def get_first_name(self):
        return self.first_name or ""

    def get_last_name(self):
        return self.last_name or ""

    def get_date_of_birth(self):
        return self.date_of_birth or ""

    def get_date_of_death(self):
        return self.date_of_death or ""

    def get_melissa_identity_key(self):
        return self.melissa_identity_key or ""


class PersonatorSearchAddress:
    def __init__(
        self, address_line_1="", suite="", city="", state="", postal_code="", plus4="", 
        melissa_address_key="", melissa_address_key_base="", move_date="",
    ):
        self.address_line_1 = address_line_1
        self.suite = suite
        self.city = city
        self.state = state
        self.postal_code = postal_code
        self.plus4 = plus4
        self.melissa_address_key = melissa_address_key
        self.melissa_address_key_base = melissa_address_key_base
        self.move_date = move_date

    @classmethod
    def from_dict(cls, data: dict):
        cls.data = data
        return cls(
            address_line_1=data.get("AddressLine1", ""),
            suite=data.get("Suite", ""),
            city=data.get("City", ""),
            state=data.get("State", ""),
            postal_code=data.get("PostalCode", ""),
            plus4=data.get("Plus4", ""),
            melissa_address_key=data.get("MelissaAddressKey", ""),
            melissa_address_key_base=data.get("MelissaAddressKeyBase", ""),
            move_date=data.get("MoveDate", ""),
        )
    
    # Setters
    def set_address_line_1(self, address_line_1):
        self.address_line_1 = address_line_1

    def set_suite(self, suite):
        self.suite = suite

    def set_city(self, city):
        self.city = city

    def set_state(self, state):
        self.state = state

    def set_postal_code(self, postal_code):
        self.postal_code = postal_code

    def set_plus4(self, plus4):
        self.plus4 = plus4

    def set_melissa_address_key(self, melissa_address_key):
        self.melissa_address_key = melissa_address_key

    def set_melissa_address_key_base(self, melissa_address_key_base):
        self.melissa_address_key_base = melissa_address_key_base

    def set_move_date(self, move_date):
        self.move_date = move_date

    # Getters
    def get_address_line_1(self):
        return self.address_line_1 or ""

    def get_suite(self):
        return self.suite or ""

    def get_city(self):
        return self.city or ""

    def get_state(self):
        return self.state or ""

    def get_postal_code(self):
        return self.postal_code or ""

    def get_plus4(self):
        return self.plus4 or ""

    def get_melissa_address_key(self):
        return self.melissa_address_key or ""

    def get_melissa_address_key_base(self):
        return self.melissa_address_key_base or ""

    def get_move_date(self):
        return self.move_date or ""

class PersonatorSearchPhone(ResponseBase):
    def __init__(self, phone_number=""):
        self.phone_number = phone_number

    @classmethod
    def from_dict(cls, data: dict):
        cls.data = data
        return cls(
            phone_number=data.get("PhoneNumber", "")
        )

    # Setter
    def set_phone_number(self, phone_number):
        self.phone_number = phone_number

    # Getter
    def get_phone_number(self):
        return self.phone_number or ""


class PersonatorSearchEmail(ResponseBase):
    def __init__(self, email=""):
        self.email = email
    
    @classmethod
    def from_dict(cls, data: dict):
        cls.data = data
        return cls(
            email=data.get("Email", "")
        )
    # Setter
    def set_email(self, email):
        self.email = email

    # Getter
    def get_email(self):
        return self.email or ""






from .ResponseBase import ResponseBase

class ReverseGeoCoderResponse(ResponseBase):

    def __init__(self, version="", transmission_results="", transmission_reference="", results="", total_records="", records=None):
        self.version = version
        self.transmission_reference = transmission_reference
        self.transmission_results = transmission_results
        self.results = results
        self.total_records = total_records
        self.records = records if records is not None else []

    @classmethod
    def populate_from_dict(cls, data: dict):
        cls.data = data
        records = [ReverseGeoCoderRecord.from_dict(record) for record in data.get("Records", [])]
        return cls(
            version=data.get("Version", ""),
            transmission_reference=data.get("TransmissionReference", ""),
            transmission_results=data.get("TransmissionResults", ""),
            results=data.get("Results", ""),
            total_records=data.get("TotalRecords", ""),
            records=records
        )

     # Setters
    def set_version(self, version):
        self.version = version

    def set_transmission_reference(self, transmission_reference):
        self.transmission_reference = transmission_reference

    def set_transmission_results(self, transmission_results):
        self.transmission_results = transmission_results

    def set_results(self, results):
        self.results = results

    def set_total_records(self, total_records):
        self.total_records = total_records

    def set_records(self, records):
        self.records = records

    # Getters
    def get_version(self):
        return self.version or ""

    def get_transmission_reference(self):
        return self.transmission_reference or ""

    def get_transmission_results(self):
        return self.transmission_results or ""

    def get_results(self):
        return self.results or ""

    def get_total_records(self):
        return self.total_records or ""

    def get_records(self):
        return self.records or []
    
class ReverseGeoCoderRecord(ResponseBase):
    def __init__(self, address_line_1="", suite_name="", suite_count="", city="", state="", postal_code="", address_key="", latitude="", 
                 longitude="", distance="", melissa_address_key="", melissa_address_key_base="", carrier_route="", laitude="", record_id=""):
        # For doLookup endpoint
        self.address_line_1 = address_line_1
        self.suite_name = suite_name
        self.suite_count = suite_count
        self.city = city
        self.state = state
        self.postal_code = postal_code
        self.address_key = address_key
        self.latitude = latitude
        self.longitude = longitude
        self.distance = distance
        self.melissa_address_key = melissa_address_key
        self.melissa_address_key_base = melissa_address_key_base

        # For doLookupPostalCodes endpoint
        self.carrier_route = carrier_route
        self.laitude = laitude

        # For doLookupFromList
        self.record_id = record_id

    @classmethod
    def from_dict(cls, data):
        cls.data = data
        return cls(
            address_line_1=data.get("AddressLine1", ""),
            suite_name=data.get("SuiteName", ""),
            suite_count=data.get("SuiteCount", ""),
            city=data.get("City", ""),
            state=data.get("State", ""),
            postal_code=data.get("PostalCode", ""),
            address_key=data.get("AddressKey", ""),
            latitude=data.get("Latitude", ""),
            longitude=data.get("Longitude", ""),
            distance=data.get("Distance", ""),
            melissa_address_key=data.get("MelissaAddressKey", ""),
            melissa_address_key_base=data.get("MelissaAddressKeyBase", ""),
            carrier_route=data.get("CarrierRoute", ""),
            laitude=data.get("Laitude", ""),
            record_id=data.get("RecordID", "")
        )
    
    # Setters
    def set_address_line_1(self, address_line_1):
        self.address_line_1 = address_line_1

    def set_suite_name(self, suite_name):
        self.suite_name = suite_name

    def set_suite_count(self, suite_count):
        self.suite_count = suite_count

    def set_city(self, city):
        self.city = city

    def set_state(self, state):
        self.state = state

    def set_postal_code(self, postal_code):
        self.postal_code = postal_code

    def set_address_key(self, address_key):
        self.address_key = address_key

    def set_latitude(self, latitude):
        self.latitude = latitude

    def set_longitude(self, longitude):
        self.longitude = longitude

    def set_distance(self, distance):
        self.distance = distance

    def set_melissa_address_key(self, melissa_address_key):
        self.melissa_address_key = melissa_address_key

    def set_melissa_address_key_base(self, melissa_address_key_base):
        self.melissa_address_key_base = melissa_address_key_base

    def set_carrier_route(self, carrier_route):
        self.carrier_route = carrier_route

    def set_laitude(self, laitude):
        self.laitude = laitude

    def set_record_id(self, record_id):
        self.record_id = record_id

    # Getters
    def get_address_line_1(self):
        return self.address_line_1 or ""

    def get_suite_name(self):
        return self.suite_name or ""

    def get_suite_count(self):
        return self.suite_count or ""

    def get_city(self):
        return self.city or ""

    def get_state(self):
        return self.state or ""

    def get_postal_code(self):
        return self.postal_code or ""

    def get_address_key(self):
        return self.address_key or ""

    def get_latitude(self):
        return self.latitude or ""

    def get_longitude(self):
        return self.longitude or ""

    def get_distance(self):
        return self.distance or ""

    def get_melissa_address_key(self):
        return self.melissa_address_key or ""

    def get_melissa_address_key_base(self):
        return self.melissa_address_key_base or ""

    def get_carrier_route(self):
        return self.carrier_route or ""

    def get_laitude(self):
        return self.laitude or ""

    def get_record_id(self):
        return self.record_id or ""






from .BusinessCoder import BusinessCoder
from .CloudApiBase import CloudApiBase
from .GlobalAddressVerification import GlobalAddressVerification
from .GlobalEmail import GlobalEmail
from .GlobalExpressEntry import GlobalExpressEntry
from .GlobalIP import GlobalIP
from .GlobalName import GlobalName
from .GlobalPhone import GlobalPhone
from .PeopleBusinessSearch import PeopleBusinessSearch
from .PersonatorConsumer import PersonatorConsumer
from .PersonatorIdentity import PersonatorIdentity
from .PersonatorSearch import PersonatorSearch
from .Property import Property
from .ReverseGeoCoder import ReverseGeoCoder
from .SmartMover import SmartMover
from .SSNNameMatch import SSNNameMatch
from .StreetRoute import StreetRoute

__all__ = [
    "BusinessCoder",
    "CloudApiBase",
    "GlobalAddressVerification",
    "GlobalEmail",
    "GlobalExpressEntry",
    "GlobalIP",
    "GlobalName",
    "GlobalPhone",
    "PeopleBusinessSearch",
    "PersonatorConsumer",
    "PersonatorIdentity",
    "PersonatorSearch",
    "Property",
    "ReverseGeoCoder",
    "SmartMover",
    "SSNNameMatch",
    "StreetRoute",
]


from abc import ABC
from typing import List, TypeVar, Generic
from .RecordRequests import *

TRecord = TypeVar('TRecord')

class PostRequestBase(ABC, Generic[TRecord]):
    def __init__(self, records: List[TRecord], transmission_reference, options):
        self.transmission_reference = transmission_reference
        self.options = options
        self.records = records

class BusinessCoderPostRequest(PostRequestBase[BusinessCoderRecordRequest]):
    def __init__(self, records: List[BusinessCoderRecordRequest], id, t=None, opt=None, cols=None):
        super().__init__(records, t, opt)
        self.id = id
        self.cols = cols
        self.opt = opt
        self.t = t


    def to_dict(self):
        return {
            "T": self.t,
            "Opt": self.opt,
            "Records": [BusinessCoderRecordRequest.to_dict(record) for record in self.records],           
            "Id": self.id,
            "Cols": self.cols
        }

class GlobalAddressVerificationPostRequest(PostRequestBase[GlobalAddressVerificationRecordRequest]):
    def __init__(self, records: List[GlobalAddressVerificationRecordRequest], customer_id, transmission_reference=None, options=None):
        super().__init__(records, transmission_reference, options)
        self.customer_id = customer_id

    def to_dict(self):
        return {
            "TransmissionReference": self.transmission_reference,
            "Options": self.options,
            "Records": [GlobalAddressVerificationRecordRequest.to_dict(record) for record in self.records],           
            "CustomerId": self.customer_id
        }

class GlobalEmailPostRequest(PostRequestBase[GlobalEmailRecordRequest]):
    def __init__(self, records: List[GlobalEmailRecordRequest], customer_id, transmission_reference=None, options=None):
        super().__init__(records, transmission_reference, options)
        self.customer_id = customer_id

    def to_dict(self):
        return {
            "TransmissionReference": self.transmission_reference,
            "Options": self.options,
            "Records": [GlobalEmailRecordRequest.to_dict(record) for record in self.records],           
            "CustomerId": self.customer_id
        }
    
class GlobalIPPostRequest(PostRequestBase[GlobalIPRecordRequest]):
    def __init__(self, records: List[GlobalIPRecordRequest], customer_id, transmission_reference=None, options=None):   
        super().__init__(records, transmission_reference, options)
        self.customer_id = customer_id

    def to_dict(self):
        return {
            "TransmissionReference": self.transmission_reference,
            "Options": self.options,
            "Records": [GlobalIPRecordRequest.to_dict(record) for record in self.records],           
            "CustomerId": self.customer_id
        }

    
class GlobalNamePostRequest(PostRequestBase[GlobalNameRecordRequest]):
    def __init__(self, records: List[GlobalNameRecordRequest], customer_id, transmission_reference=None, options=None):   
        super().__init__(records, transmission_reference, options)
        self.customer_id = customer_id

    def to_dict(self):
        return {
            "TransmissionReference": self.transmission_reference,
            "Options": self.options,
            "Records": [GlobalNameRecordRequest.to_dict(record) for record in self.records],
            "CustomerId": self.customer_id
        }
    
class GlobalPhonePostRequest(PostRequestBase[GlobalPhoneRecordRequest]):
    def __init__(self, records: List[GlobalPhoneRecordRequest], customer_id, transmission_reference=None, options=None):   
        super().__init__(records, transmission_reference, options)
        self.customer_id = customer_id

    def to_dict(self):
        return {
            "TransmissionReference": self.transmission_reference,
            "Options": self.options,
            "Records": [GlobalPhoneRecordRequest.to_dict(record) for record in self.records],
            "CustomerId": self.customer_id
        }
    
class PersonatorConsumerPostRequest(PostRequestBase[PersonatorConsumerRecordRequest]):
    def __init__(self, records: List[PersonatorConsumerRecordRequest], customer_id, transmission_reference=None, options=None):   
        super().__init__(records, transmission_reference, options)
        self.customer_id = customer_id

    def to_dict(self):
        return {
            "TransmissionReference": self.transmission_reference,
            "Options": self.options,
            "Records": [PersonatorConsumerRecordRequest.to_dict(record) for record in self.records],
            "CustomerId": self.customer_id
        }
    
class PropertyPostRequest(PostRequestBase[PropertyRecordRequest]):
    def __init__(
        self, customer_id, records: List[PropertyRecordRequest], transmission_reference=None, options=None,
        columns="", total_records="", apn="", fips="", free_form="", property_mak="", owner_mak=""
    ):
        super().__init__(records, transmission_reference, options)

        self.customer_id = customer_id
        self.columns = columns
        self.total_records = total_records
        self.apn = apn
        self.fips = fips
        self.free_form = free_form
        self.property_mak = property_mak
        self.owner_mak = owner_mak

    def to_dict(self):
        return {
            "Records": [PropertyRecordRequest.to_dict(record) for record in self.records],
            "CustomerId": self.customer_id,
            "Columns": self.columns,
            "TotalRecords": self.total_records,
            "APN": self.apn,
            "FIPS": self.fips,
            "FreeForm": self.free_form,
            "PropertyMAK": self.property_mak,
            "OwnerMAK": self.owner_mak
        }
    

class SmartMoverPostRequest(PostRequestBase[SmartMoverRecordRequest]):
    def __init__(self, records: List[SmartMoverRecordRequest], customer_id, transmission_reference=None, options=None, job_id=None, 
                 paf_id=None, execution_id=None, actions=None, columns=None, opt_smart_mover_list_name=None):
        super().__init__(records, transmission_reference, options)
        self.customer_id = customer_id
        self.job_id = job_id
        self.paf_id = paf_id
        self.execution_id = execution_id
        self.actions = actions
        self.columns = columns
        self.opt_smart_mover_list_name = opt_smart_mover_list_name


    def to_dict(self):
        return {
            "Records": [SmartMoverRecordRequest.to_dict(record) for record in self.records],
            "CustomerID": self.customer_id,
            "TransmissionReference": self.transmission_reference,
            "Options": self.options,
            "JobID": self.job_id,
            "PafID": self.paf_id,
            "ExecutionID": self.execution_id,
            "Actions": self.actions,
            "Columns": self.columns,
            "OptSmartMoverListName": self.opt_smart_mover_list_name
        }

class SSNNameMatchPostRequest(PostRequestBase[SSNNameMatchRecordRequest]):
    def __init__(self, records: List[SSNNameMatchRecordRequest], customer_id, transmission_reference=None, options=None):
        super().__init__(records, transmission_reference, options)
        self.customer_id = customer_id

    def to_dict(self):
        return {
            "TransmissionReference": self.transmission_reference,
            "Options": self.options,
            "Records": [SSNNameMatchRecordRequest.to_dict(record) for record in self.records],           
            "CustomerId": self.customer_id
        }

class StreetRoutePostRequest(PostRequestBase[StreetRouteRecordRequest]):
    def __init__(self, records: List[StreetRouteRecordRequest], customer_id, transmission_reference=None, options=None):
        super().__init__(records, transmission_reference, options)
        self.customer_id = customer_id

    def to_dict(self):
        return {
            "TransmissionReference": self.transmission_reference,
            "Options": self.options,
            "Records": [StreetRouteRecordRequest.to_dict(record) for record in self.records],           
            "CustomerId": self.customer_id
        }







import json
import os
from .CloudApiBase import CloudApiBase
from ..PostReqestBase import GlobalAddressVerificationPostRequest

class GlobalAddressVerification(CloudApiBase):
    def __init__(self, license=None):
        
        super().__init__("https://address.melissadata.net", "/v3/WEB/GlobalAddress/doGlobalAddress")
        self.license = license or os.environ.get("MD_LICENSE")
        self.parameters["format"] = "json"
        self.records = []
        self.post_body = None

        self.parameter_mappings = {
            "organization": "org",
            "last_name": "last",
            "address_line_1": "a1",
            "address_line_2": "a2",
            "address_line_3": "a3",
            "address_line_4": "a4",
            "address_line_5": "a5",
            "address_line_6": "a6",
            "address_line_7": "a7",
            "address_line_8": "a8",
            "double_dependent_locality": "ddeploc",
            "dependent_locality": "deploc",
            "locality": "loc",
            "administrative_area": "admarea",
            "postal": "postal",
            "sub_national_area": "subnatarea",
            "country": "ctry",
            "transmission_reference": "t",
            "opt": "opt"
        }

        # Initialize private backing fields
        for attr in self.parameter_mappings.keys():
            setattr(self, f"_{attr}", None)

    def _set_parameter(self, attr_name, value):
        """
        Helper method to set a private attribute and update the parameters dictionary.
        """
        
        setattr(self, f"_{attr_name}", value)
        self.parameters[self.parameter_mappings[attr_name]] = value

    # Properties
    @property
    def organization(self):
        return self._organization

    @organization.setter
    def organization(self, value):
        self._set_parameter("organization", value)

    @property
    def last_name(self):
        return self._last_name

    @last_name.setter
    def last_name(self, value):
        self._set_parameter("last_name", value)

    @property
    def address_line_1(self):
        return self._address_line_1

    @address_line_1.setter
    def address_line_1(self, value):
        self._set_parameter("address_line_1", value)

    @property
    def address_line_2(self):
        return self._address_line_2

    @address_line_2.setter
    def address_line_2(self, value):
        self._set_parameter("address_line_2", value)

    @property
    def address_line_3(self):
        return self._address_line_3

    @address_line_3.setter
    def address_line_3(self, value):
        self._set_parameter("address_line_3", value)

    @property
    def address_line_4(self):
        return self._address_line_4

    @address_line_4.setter
    def address_line_4(self, value):
        self._set_parameter("address_line_4", value)

    @property
    def address_line_5(self):
        return self._address_line_5

    @address_line_5.setter
    def address_line_5(self, value):
        self._set_parameter("address_line_5", value)

    @property
    def address_line_6(self):
        return self._address_line_6

    @address_line_6.setter
    def address_line_6(self, value):
        self._set_parameter("address_line_6", value)

    @property
    def address_line_7(self):
        return self._address_line_7

    @address_line_7.setter
    def address_line_7(self, value):
        self._set_parameter("address_line_7", value)

    @property
    def address_line_8(self):
        return self._address_line_8

    @address_line_8.setter
    def address_line_8(self, value):
        self._set_parameter("address_line_8", value)

    @property
    def double_dependent_locality(self):
        return self._double_dependent_locality

    @double_dependent_locality.setter
    def double_dependent_locality(self, value):
        self._set_parameter("double_dependent_locality", value)

    @property
    def dependent_locality(self):
        return self._dependent_locality

    @dependent_locality.setter
    def dependent_locality(self, value):
        self._set_parameter("dependent_locality", value)

    @property
    def locality(self):
        return self._locality

    @locality.setter
    def locality(self, value):
        self._set_parameter("locality", value)

    @property
    def administrative_area(self):
        return self._administrative_area

    @administrative_area.setter
    def administrative_area(self, value):
        self._set_parameter("administrative_area", value)

    @property
    def postal(self):
        return self._postal

    @postal.setter
    def postal(self, value):
        self._set_parameter("postal", value)

    @property
    def sub_national_area(self):
        return self._sub_national_area

    @sub_national_area.setter
    def sub_national_area(self, value):
        self._set_parameter("sub_national_area", value)

    @property
    def country(self):
        return self._country

    @country.setter
    def country(self, value):
        self._set_parameter("country", value)

    @property
    def transmission_reference(self):
        return self._transmission_reference

    @transmission_reference.setter
    def transmission_reference(self, value):
        self._set_parameter("transmission_reference", value)

    @property
    def opt(self):
        return self._opt

    @opt.setter
    def opt(self, value):
        self._set_parameter("opt", value)

    # Setters

    def set_organization(self, organization):
        self._set_parameter("organization", organization)

    def set_last_name(self, last_name):
        self._set_parameter("last_name", last_name)

    def set_address_line_1(self, address_line_1):
        self._set_parameter("address_line_1", address_line_1)

    def set_address_line_2(self, address_line_2):
        self._set_parameter("address_line_2", address_line_2)

    def set_address_line_3(self, address_line_3):
        self._set_parameter("address_line_3", address_line_3)

    def set_address_line_4(self, address_line_4):
        self._set_parameter("address_line_4", address_line_4)

    def set_address_line_5(self, address_line_5):
        self._set_parameter("address_line_5", address_line_5)

    def set_address_line_6(self, address_line_6):
        self._set_parameter("address_line_6", address_line_6)

    def set_address_line_7(self, address_line_7):
        self._set_parameter("address_line_7", address_line_7)

    def set_address_line_8(self, address_line_8):
        self._set_parameter("address_line_8", address_line_8)

    def set_double_dependent_locality(self, double_dependent_locality):
        self._set_parameter("double_dependent_locality", double_dependent_locality)

    def set_dependent_locality(self, dependent_locality):
        self._set_parameter("dependent_locality", dependent_locality)

    def set_locality(self, locality):
        self._set_parameter("locality", locality)

    def set_administrative_area(self, administrative_area):
        self._set_parameter("administrative_area", administrative_area)

    def set_postal(self, postal):
        self._set_parameter("postal", postal)

    def set_sub_national_area(self, sub_national_area):
        self._set_parameter("sub_national_area", sub_national_area)

    def set_country(self, country):
        self._set_parameter("country", country)

    def set_transmission_reference(self, transmission_reference):
        self._set_parameter("transmission_reference", transmission_reference)

    def set_opt(self, opt):
        self._set_parameter("opt", opt)

    def set_post_body(self, post_body):
        self.post_body = post_body

    def set_value(self, parameter, value):
        parameter = parameter.strip() if parameter else None
        
        if parameter in self.parameter_mappings:
            parameter_key = self.parameter_mappings[parameter]
            
            property_name = next((key for key, val in self.parameter_mappings.items() 
                                  if val.lower() == parameter_key.lower()), None)
            
            if property_name:
                if hasattr(self, property_name):
                    setattr(self, property_name, value)
        else:
            # If not in derived class, go to base class implementation
            super().set_value(parameter, value)

    # Getters

    def get_organization(self):
        return self.parameters.get("org", "")

    def get_last_name(self):
        return self.parameters.get("last", "")

    def get_address_line_1(self):
        return self.parameters.get("a1", "")

    def get_address_line_2(self):
        return self.parameters.get("a2", "")

    def get_address_line_3(self):
        return self.parameters.get("a3", "")

    def get_address_line_4(self):
        return self.parameters.get("a4", "")

    def get_address_line_5(self):
        return self.parameters.get("a5", "")

    def get_address_line_6(self):
        return self.parameters.get("a6", "")

    def get_address_line_7(self):
        return self.parameters.get("a7", "")

    def get_address_line_8(self):
        return self.parameters.get("a8", "")

    def get_double_dependent_locality(self):
        return self.parameters.get("ddeploc", "")

    def get_dependent_locality(self):
        return self.parameters.get("deploc", "")

    def get_locality(self):
        return self.parameters.get("loc", "")

    def get_administrative_area(self):
        return self.parameters.get("admarea", "")

    def get_postal(self):
        return self.parameters.get("postal", "")

    def get_sub_national_area(self):
        return self.parameters.get("subnatarea", "")

    def get_country(self):
        return self.parameters.get("ctry", "")

    def get_transmission_reference(self):
        return self.parameters.get("t", "")

    def get_opt(self):
        return self.parameters.get("opt", "")

    def get_post_body(self):
        return self.post_body

    # Class Methods

    def add_record(self, record):
        self.records.append(record)

    def clear_records(self):
        self.records.clear()

    def post(self, response_type):
        """
        This synchronous function makes the batch request using the post body.

        Returns:
            A string or deserialized json response object.
        """
        if self.records and len(self.records) > 0:
            post_request = GlobalAddressVerificationPostRequest(
                transmission_reference=self.get_transmission_reference(),
                options=self.get_opt(),
                records=self.records,
                customer_id=self.get_license()
            )
            response = self.send_post_request(post_request)
        else:
            response = self.send_post_request(self.post_body)

        return self.handle_response(response, response_type)


    def get_api_version(self):
        """
        Makes a synchronous getversion request and parses the response for the API version.

        Returns:
            The Cloud API version.
        """
        json_response = self.send_version_request()

        response_objectect = json.loads(json_response)

        global_address_verification_version = response_objectect.get("Version", "")

        return global_address_verification_version




from .ResponseBase import ResponseBase

class SSNNameMatchResponse(ResponseBase):
    def __init__(self, version="", transmission_reference="", transmission_results="", total_records="", records=None):
        self.version = version
        self.transmission_reference = transmission_reference
        self.transmission_results = transmission_results
        self.total_records = total_records
        self.records = records if records is not None else []

    @classmethod
    def populate_from_dict(cls, data: dict):
        cls.data = data
        records = [SSNNameMatchRecord.from_dict(record) for record in data.get("Records", [])]
        return cls(
            version=data.get("Version", ""),
            transmission_reference=data.get("TransmissionReference", ""),
            transmission_results=data.get("TransmissionResults", ""),
            total_records = data.get("TotalRecords", ""),
            records=records
        )
    
    # Setters
    def set_version(self, version):
        self.version = version

    def set_transmission_reference(self, transmission_reference):
        self.transmission_reference = transmission_reference

    def set_transmission_results(self, transmission_results):
        self.transmission_results = transmission_results

    def set_total_records(self, total_records):
        self.total_records = total_records

    # Getters
    def get_version(self):
        return self.version or ""

    def get_transmission_reference(self):
        return self.transmission_reference or ""

    def get_transmission_results(self):
        return self.transmission_results or ""

    def get_total_records(self):
        return self.total_records or ""
    

class SSNNameMatchRecord:
    def __init__(
        self,
        record_id="",
        ssn="",
        issuing_state="",
        results="",
        results_from_data_source="",
    ):
        self.record_id = record_id
        self.ssn = ssn
        self.issuing_state = issuing_state
        self.results = results
        self.results_from_data_source = results_from_data_source

    @classmethod
    def from_dict(cls, data):
        cls.data = data
        return cls(
            record_id=data.get("RecordID", ""),
            ssn=data.get("SSN", ""),
            issuing_state=data.get("IssuingState", ""),
            results=data.get("Results", ""),
            results_from_data_source=data.get("ResultsFromDataSource", ""),
        )
    
    # Setters
    def set_record_id(self, record_id):
        self._record_id = record_id

    def set_ssn(self, ssn):
        self._ssn = ssn

    def set_issuing_state(self, issuing_state):
        self._issuing_state = issuing_state

    def set_results(self, results):
        self._results = results

    def set_results_from_data_source(self, results_from_data_source):
        self.results_from_data_source = results_from_data_source

    # Getters
    def get_record_id(self):
        return self.record_id or ""

    def get_ssn(self):
        return self.ssn or ""

    def get_issuing_state(self):
        return self.issuing_state or ""

    def get_results(self):
        return self.results or ""

    def get_results_from_data_source(self):
        return self.results_from_data_source or ""









from setuptools import setup, find_packages


with open("PyPIREADME.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setup(
    name='melissadatacloudapi',
    version='3.13.122', #change in the readme badges too
    packages=find_packages(where='.'),
    install_requires=[],  # List any dependencies here
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/MelissaData/MelissaCloudAPI-Python3",
)



from .ResponseBase import ResponseBase

class GlobalExpressEntryResponse(ResponseBase):

    def __init__(self, version="", result_code="", error_string="", results=None):
        self.version = version
        self.result_code = result_code
        self.error_string = error_string
        self.results = results if results is not None else []

    @classmethod
    def populate_from_dict(cls, data: dict):
        cls.data=data
        results = [GlobalExpressEntryRecord.from_dict(result) for result in data.get("Results", [])]
        return cls(
            version=data.get("Version", ""),
            result_code=data.get("ResultCode", ""),
            error_string=data.get("ErrorString", ""),
            results=results
        )

    # Setters
    def set_version(self, version):
        self.version = version

    def set_result_code(self, result_code):
        self.result_code = result_code

    def set_error_string(self, error_string):
        self.error_string = error_string

    # Getters
    def get_version(self):
        return self.version if self.version else ""

    def get_result_code(self):
        return self.result_code if self.result_code else ""

    def get_error_string(self):
        return self.error_string if self.error_string else ""
    
class GlobalExpressEntryRecord(ResponseBase):
    def __init__(self, address=None, country="", english="", spanish="", french="", german="", simplified_chinese="", char2_iso="", char3_iso="", iso_numeric=""):
        # For GlobalExpressAddress endpoint
        
        self.address = address  
        
        # For GlobalExpressCountry endpoint
        self.country = country
        self.english = english
        self.spanish = spanish
        self.french = french
        self.german = german
        self.simplified_chinese = simplified_chinese
        self.char2_iso = char2_iso
        self.char3_iso = char3_iso
        self.iso_numeric = iso_numeric

    @classmethod
    def from_dict(cls, data):
        cls.data=data
        return cls(
            address = GlobalExpressEntryAddress.from_dict(data.get("Address")) if data.get("Address") is not None else None,
            country=data.get("Country", ""),
            english=data.get("English", ""),
            spanish=data.get("Spanish", ""),
            french=data.get("French", ""),
            german=data.get("German", ""),
            simplified_chinese=data.get("SimplifiedChinese", ""),
            char2_iso=data.get("Char2ISO", ""),
            char3_iso=data.get("Char3ISO", ""),
            iso_numeric=data.get("ISONumeric", "")
        )
    
    # Setters
    def set_country(self, country):
        self.country = country

    def set_english(self, english):
        self.english = english

    def set_spanish(self, spanish):
        self.spanish = spanish

    def set_french(self, french):
        self.french = french

    def set_german(self, german):
        self.german = german

    def set_simplified_chinese(self, simplified_chinese):
        self.simplified_chinese = simplified_chinese

    def set_char2_iso(self, char2_iso):
        self.char2_iso = char2_iso

    def set_char3_iso(self, char3_iso):
        self.char3_iso = char3_iso

    def set_iso_numeric(self, iso_numeric):
        self.iso_numeric = iso_numeric

    # Getters
    def get_country(self):
        return self.country or ""

    def get_english(self):
        return self.english or ""

    def get_spanish(self):
        return self.spanish or ""

    def get_french(self):
        return self.french or ""

    def get_german(self):
        return self.german or ""

    def get_simplified_chinese(self):
        return self.simplified_chinese or ""

    def get_char2_iso(self):
        return self.char2_iso or ""

    def get_char3_iso(self):
        return self.char3_iso or ""

    def get_iso_numeric(self):
        return self.iso_numeric or ""

class GlobalExpressEntryAddress(ResponseBase):
    def __init__(self, address="", address1="", address2="", address3="", address4="", address5="", address6="",
                 address7="", address8="", address9="", address10="", address11="", address12="", delivery_address="",
                 delivery_address1="", delivery_address2="", delivery_address3="", delivery_address4="", delivery_address5="",
                 delivery_address6="", delivery_address7="", delivery_address8="", delivery_address9="", delivery_address10="",
                 delivery_address11="", delivery_address12="", country_name="", iso3166_2="", iso3166_3="", iso3166_n="",
                 super_administrative_area="", administrative_area="", sub_administrative_area="", locality="",
                 city_accepted="", city_not_accepted="", dependent_locality="", double_dependent_locality="", thoroughfare="",
                 dependent_thoroughfare="", building="", premise="", sub_building="", postal_code="", postal_code_primary="",
                 postal_code_secondary="", organization="", post_box="", unmatched="", general_delivery="", delivery_installation="",
                 route="", additional_content="", country_subdivision_code="", mak="", base_mak="", latitude="", longitude="",
                 distance_from_point=""):
        
        self.address = address
        self.address1 = address1
        self.address2 = address2
        self.address3 = address3
        self.address4 = address4
        self.address5 = address5
        self.address6 = address6
        self.address7 = address7
        self.address8 = address8
        self.address9 = address9
        self.address10 = address10
        self.address11 = address11
        self.address12 = address12
        self.delivery_address = delivery_address
        self.delivery_address1 = delivery_address1
        self.delivery_address2 = delivery_address2
        self.delivery_address3 = delivery_address3
        self.delivery_address4 = delivery_address4
        self.delivery_address5 = delivery_address5
        self.delivery_address6 = delivery_address6
        self.delivery_address7 = delivery_address7
        self.delivery_address8 = delivery_address8
        self.delivery_address9 = delivery_address9
        self.delivery_address10 = delivery_address10
        self.delivery_address11 = delivery_address11
        self.delivery_address12 = delivery_address12
        self.country_name = country_name
        self.iso3166_2 = iso3166_2
        self.iso3166_3 = iso3166_3
        self.iso3166_n = iso3166_n
        self.super_administrative_area = super_administrative_area
        self.administrative_area = administrative_area
        self.sub_administrative_area = sub_administrative_area
        self.locality = locality
        self.city_accepted = city_accepted
        self.city_not_accepted = city_not_accepted
        self.dependent_locality = dependent_locality
        self.double_dependent_locality = double_dependent_locality
        self.thoroughfare = thoroughfare
        self.dependent_thoroughfare = dependent_thoroughfare
        self.building = building
        self.premise = premise
        self.sub_building = sub_building
        self.postal_code = postal_code
        self.postal_code_primary = postal_code_primary
        self.postal_code_secondary = postal_code_secondary
        self.organization = organization
        self.post_box = post_box
        self.unmatched = unmatched
        self.general_delivery = general_delivery
        self.delivery_installation = delivery_installation
        self.route = route
        self.additional_content = additional_content
        self.country_subdivision_code = country_subdivision_code
        self.mak = mak
        self.base_mak = base_mak
        self.latitude = latitude
        self.longitude = longitude
        self.distance_from_point = distance_from_point

    @classmethod
    def from_dict(cls, data):
        cls.data = data
        return cls(
            address=data.get("Address", ""),
            address1=data.get("Address1", ""),
            address2=data.get("Address2", ""),
            address3=data.get("Address3", ""),
            address4=data.get("Address4", ""),
            address5=data.get("Address5", ""),
            address6=data.get("Address6", ""),
            address7=data.get("Address7", ""),
            address8=data.get("Address8", ""),
            address9=data.get("Address9", ""),
            address10=data.get("Address10", ""),
            address11=data.get("Address11", ""),
            address12=data.get("Address12", ""),
            delivery_address=data.get("DeliveryAddress", ""),
            delivery_address1=data.get("DeliveryAddress1", ""),
            delivery_address2=data.get("DeliveryAddress2", ""),
            delivery_address3=data.get("DeliveryAddress3", ""),
            delivery_address4=data.get("DeliveryAddress4", ""),
            delivery_address5=data.get("DeliveryAddress5", ""),
            delivery_address6=data.get("DeliveryAddress6", ""),
            delivery_address7=data.get("DeliveryAddress7", ""),
            delivery_address8=data.get("DeliveryAddress8", ""),
            delivery_address9=data.get("DeliveryAddress9", ""),
            delivery_address10=data.get("DeliveryAddress10", ""),
            delivery_address11=data.get("DeliveryAddress11", ""),
            delivery_address12=data.get("DeliveryAddress12", ""),
            country_name=data.get("CountryName", ""),
            iso3166_2=data.get("ISO3166_2", ""),
            iso3166_3=data.get("ISO3166_3", ""),
            iso3166_n=data.get("ISO3166_N", ""),
            super_administrative_area=data.get("SuperAdministrativeArea", ""),
            administrative_area=data.get("AdministrativeArea", ""),
            sub_administrative_area=data.get("SubAdministrativeArea", ""),
            locality=data.get("Locality", ""),
            city_accepted=data.get("CityAccepted", ""),
            city_not_accepted=data.get("CityNotAccepted", ""),
            dependent_locality=data.get("DependentLocality", ""),
            double_dependent_locality=data.get("DoubleDependentLocality", ""),
            thoroughfare=data.get("Thoroughfare", ""),
            dependent_thoroughfare=data.get("DependentThoroughfare", ""),
            building=data.get("Building", ""),
            premise=data.get("Premise", ""),
            sub_building=data.get("SubBuilding", ""),
            postal_code=data.get("PostalCode", ""),
            postal_code_primary=data.get("PostalCodePrimary", ""),
            postal_code_secondary=data.get("PostalCodeSecondary", ""),
            organization=data.get("Organization", ""),
            post_box=data.get("PostBox", ""),
            unmatched=data.get("Unmatched", ""),
            general_delivery=data.get("GeneralDelivery", ""),
            delivery_installation=data.get("DeliveryInstallation", ""),
            route=data.get("Route", ""),
            additional_content=data.get("AdditionalContent", ""),
            country_subdivision_code=data.get("CountrySubdivisionCode", ""),
            mak=data.get("MAK", ""),
            base_mak=data.get("BaseMAK", ""),
            latitude=data.get("Latitude", ""),
            longitude=data.get("Longitude", ""),
            distance_from_point=data.get("DistanceFromPoint", "")
        )
    
    # Setters
    def set_address(self, address):
        self.address = address

    def set_address1(self, address1):
        self.address1 = address1

    def set_address2(self, address2):
        self.address2 = address2

    def set_address3(self, address3):
        self.address3 = address3

    def set_address4(self, address4):
        self.address4 = address4

    def set_address5(self, address5):
        self.address5 = address5

    def set_address6(self, address6):
        self.address6 = address6

    def set_address7(self, address7):
        self.address7 = address7

    def set_address8(self, address8):
        self.address8 = address8

    def set_address9(self, address9):
        self.address9 = address9

    def set_address10(self, address10):
        self.address10 = address10

    def set_address11(self, address11):
        self.address11 = address11

    def set_address12(self, address12):
        self.address12 = address12

    def set_delivery_address(self, delivery_address):
        self.delivery_address = delivery_address

    def set_delivery_address1(self, delivery_address1):
        self.delivery_address1 = delivery_address1

    def set_delivery_address2(self, delivery_address2):
        self.delivery_address2 = delivery_address2

    def set_delivery_address3(self, delivery_address3):
        self.delivery_address3 = delivery_address3

    def set_delivery_address4(self, delivery_address4):
        self.delivery_address4 = delivery_address4

    def set_delivery_address5(self, delivery_address5):
        self.delivery_address5 = delivery_address5

    def set_delivery_address6(self, delivery_address6):
        self.delivery_address6 = delivery_address6

    def set_delivery_address7(self, delivery_address7):
        self.delivery_address7 = delivery_address7

    def set_delivery_address8(self, delivery_address8):
        self.delivery_address8 = delivery_address8

    def set_delivery_address9(self, delivery_address9):
        self.delivery_address9 = delivery_address9

    def set_delivery_address10(self, delivery_address10):
        self.delivery_address10 = delivery_address10

    def set_delivery_address11(self, delivery_address11):
        self.delivery_address11 = delivery_address11

    def set_delivery_address12(self, delivery_address12):
        self.delivery_address12 = delivery_address12

    def set_country_name(self, country_name):
        self.country_name = country_name

    def set_iso3166_2(self, iso3166_2):
        self.iso3166_2 = iso3166_2

    def set_iso3166_3(self, iso3166_3):
        self.iso3166_3 = iso3166_3

    def set_iso3166_n(self, iso3166_n):
        self.iso3166_n = iso3166_n

    def set_super_administrative_area(self, super_administrative_area):
        self.super_administrative_area = super_administrative_area

    def set_administrative_area(self, administrative_area):
        self.administrative_area = administrative_area

    def set_sub_administrative_area(self, sub_administrative_area):
        self.sub_administrative_area = sub_administrative_area

    def set_locality(self, locality):
        self.locality = locality

    def set_city_accepted(self, city_accepted):
        self.city_accepted = city_accepted

    def set_city_not_accepted(self, city_not_accepted):
        self.city_not_accepted = city_not_accepted

    def set_dependent_locality(self, dependent_locality):
        self.dependent_locality = dependent_locality

    def set_double_dependent_locality(self, double_dependent_locality):
        self.double_dependent_locality = double_dependent_locality

    def set_thoroughfare(self, thoroughfare):
        self.thoroughfare = thoroughfare

    def set_dependent_thoroughfare(self, dependent_thoroughfare):
        self.dependent_thoroughfare = dependent_thoroughfare

    def set_building(self, building):
        self.building = building

    def set_premise(self, premise):
        self.premise = premise

    def set_sub_building(self, sub_building):
        self.sub_building = sub_building

    def set_postal_code(self, postal_code):
        self.postal_code = postal_code

    def set_postal_code_primary(self, postal_code_primary):
        self.postal_code_primary = postal_code_primary

    def set_postal_code_secondary(self, postal_code_secondary):
        self.postal_code_secondary = postal_code_secondary

    def set_organization(self, organization):
        self.organization = organization

    def set_post_box(self, post_box):
        self.post_box = post_box

    def set_unmatched(self, unmatched):
        self.unmatched = unmatched

    def set_general_delivery(self, general_delivery):
        self.general_delivery = general_delivery

    def set_delivery_installation(self, delivery_installation):
        self.delivery_installation = delivery_installation

    def set_route(self, route):
        self.route = route

    def set_additional_content(self, additional_content):
        self.additional_content = additional_content

    def set_country_subdivision_code(self, country_subdivision_code):
        self.country_subdivision_code = country_subdivision_code

    def set_mak(self, mak):
        self.mak = mak

    def set_base_mak(self, base_mak):
        self.base_mak = base_mak

    def set_latitude(self, latitude):
        self.latitude = latitude

    def set_longitude(self, longitude):
        self.longitude = longitude

    def set_distance_from_point(self, distance_from_point):
        self.distance_from_point = distance_from_point

    # Getters
    def get_address(self):
        return self.address or ""

    def get_address1(self):
        return self.address1 or ""

    def get_address2(self):
        return self.address2 or ""

    def get_address3(self):
        return self.address3 or ""

    def get_address4(self):
        return self.address4 or ""

    def get_address5(self):
        return self.address5 or ""

    def get_address6(self):
        return self.address6 or ""

    def get_address7(self):
        return self.address7 or ""

    def get_address8(self):
        return self.address8 or ""

    def get_address9(self):
        return self.address9 or ""

    def get_address10(self):
        return self.address10 or ""

    def get_address11(self):
        return self.address11 or ""

    def get_address12(self):
        return self.address12 or ""

    def get_delivery_address(self):
        return self.delivery_address or ""

    def get_delivery_address1(self):
        return self.delivery_address1 or ""

    def get_delivery_address2(self):
        return self.delivery_address2 or ""

    def get_delivery_address3(self):
        return self.delivery_address3 or ""

    def get_delivery_address4(self):
        return self.delivery_address4 or ""

    def get_delivery_address5(self):
        return self.delivery_address5 or ""

    def get_delivery_address6(self):
        return self.delivery_address6 or ""

    def get_delivery_address7(self):
        return self.delivery_address7 or ""

    def get_delivery_address8(self):
        return self.delivery_address8 or ""

    def get_delivery_address9(self):
        return self.delivery_address9 or ""

    def get_delivery_address10(self):
        return self.delivery_address10 or ""

    def get_delivery_address11(self):
        return self.delivery_address11 or ""

    def get_delivery_address12(self):
        return self.delivery_address12 or ""

    def get_country_name(self):
        return self.country_name or ""

    def get_iso3166_2(self):
        return self.iso3166_2 or ""

    def get_iso3166_3(self):
        return self.iso3166_3 or ""

    def get_iso3166_n(self):
        return self.iso3166_n or ""

    def get_super_administrative_area(self):
        return self.super_administrative_area or ""

    def get_administrative_area(self):
        return self.administrative_area or ""

    def get_sub_administrative_area(self):
        return self.sub_administrative_area or ""

    def get_locality(self):
        return self.locality or ""

    def get_city_accepted(self):
        return self.city_accepted or ""

    def get_city_not_accepted(self):
        return self.city_not_accepted or ""

    def get_dependent_locality(self):
        return self.dependent_locality or ""

    def get_double_dependent_locality(self):
        return self.double_dependent_locality or ""

    def get_thoroughfare(self):
        return self.thoroughfare or ""

    def get_dependent_thoroughfare(self):
        return self.dependent_thoroughfare or ""

    def get_building(self):
        return self.building or ""

    def get_premise(self):
        return self.premise or ""

    def get_sub_building(self):
        return self.sub_building or ""

    def get_postal_code(self):
        return self.postal_code or ""

    def get_postal_code_primary(self):
        return self.postal_code_primary or ""

    def get_postal_code_secondary(self):
        return self.postal_code_secondary or ""

    def get_organization(self):
        return self.organization or ""

    def get_post_box(self):
        return self.post_box or ""

    def get_unmatched(self):
        return self.unmatched or ""

    def get_general_delivery(self):
        return self.general_delivery or ""

    def get_delivery_installation(self):
        return self.delivery_installation or ""

    def get_route(self):
        return self.route or ""

    def get_additional_content(self):
        return self.additional_content or ""

    def get_country_subdivision_code(self):
        return self.country_subdivision_code or ""

    def get_mak(self):
        return self.mak or ""

    def get_base_mak(self):
        return self.base_mak or ""

    def get_latitude(self):
        return self.latitude or ""

    def get_longitude(self):
        return self.longitude or ""

    def get_distance_from_point(self):
        return self.distance_from_point or ""



    



import json
import os
from .CloudApiBase import CloudApiBase
from ..PostReqestBase import PersonatorConsumerPostRequest


class PersonatorConsumer(CloudApiBase):

    def __init__(self, license=None):
        super().__init__("https://personator.melissadata.net", "/v3/WEB/ContactVerify/doContactVerify")
        self.license = license or os.environ.get("MD_LICENSE")
        self.parameters["format"] = "json"
        self.records = []
        self.post_body = None

        self.parameter_mappings = {
            "action": "act",
            "address_line_1": "a1",
            "address_line_2": "a2",
            "birth_day": "bday",
            "birth_month": "bmo",
            "birth_year": "byr",
            "city": "city",
            "company": "comp",
            "country": "ctry",
            "email": "email",
            "first_name": "first",
            "free_form": "ff",
            "full_name": "full",
            "ip": "ip",
            "last_line": "lastline",
            "last_name": "last",
            "mak": "mak",
            "mik": "MIK",
            "phone": "phone",
            "postal": "postal",
            "ssn": "ss",
            "state": "state",
            "transmission_reference": "t",
            "opt": "opt",
            "cols": "cols"
        }

        self.parameters = {}

        # Initialize private backing fields
        for attr in self.parameter_mappings.keys():
            setattr(self, f"_{attr}", None)

    def _set_parameter(self, attr_name, value):
        """
        Helper method to set a private attribute and update the parameters dictionary.
        """
        setattr(self, f"_{attr_name}", value)
        self.parameters[self.parameter_mappings[attr_name]] = value

    # Properties
    @property
    def action(self):
        return self._action

    @action.setter
    def action(self, value):
        self._set_parameter("action", value)

    @property
    def address_line_1(self):
        return self._address_line_1

    @address_line_1.setter
    def address_line_1(self, value):
        self._set_parameter("address_line_1", value)

    @property
    def address_line_2(self):
        return self._address_line_2

    @address_line_2.setter
    def address_line_2(self, value):
        self._set_parameter("address_line_2", value)

    @property
    def birth_day(self):
        return self._birth_day

    @birth_day.setter
    def birth_day(self, value):
        self._set_parameter("birth_day", value)

    @property
    def birth_month(self):
        return self._birth_month

    @birth_month.setter
    def birth_month(self, value):
        self._set_parameter("birth_month", value)

    @property
    def birth_year(self):
        return self._birth_year

    @birth_year.setter
    def birth_year(self, value):
        self._set_parameter("birth_year", value)

    @property
    def city(self):
        return self._city

    @city.setter
    def city(self, value):
        self._set_parameter("city", value)

    @property
    def company(self):
        return self._company

    @company.setter
    def company(self, value):
        self._set_parameter("company", value)

    @property
    def country(self):
        return self._country

    @country.setter
    def country(self, value):
        self._set_parameter("country", value)

    @property
    def email(self):
        return self._email

    @email.setter
    def email(self, value):
        self._set_parameter("email", value)

    @property
    def first_name(self):
        return self._first_name

    @first_name.setter
    def first_name(self, value):
        self._set_parameter("first_name", value)

    @property
    def free_form(self):
        return self._free_form

    @free_form.setter
    def free_form(self, value):
        self._set_parameter("free_form", value)

    @property
    def full_name(self):
        return self._full_name

    @full_name.setter
    def full_name(self, value):
        self._set_parameter("full_name", value)

    @property
    def ip(self):
        return self._ip

    @ip.setter
    def ip(self, value):
        self._set_parameter("ip", value)

    @property
    def last_line(self):
        return self._last_line

    @last_line.setter
    def last_line(self, value):
        self._set_parameter("last_line", value)

    @property
    def last_name(self):
        return self._last_name

    @last_name.setter
    def last_name(self, value):
        self._set_parameter("last_name", value)

    @property
    def mak(self):
        return self._mak

    @mak.setter
    def mak(self, value):
        self._set_parameter("mak", value)

    @property
    def mik(self):
        return self._mik

    @mik.setter
    def mik(self, value):
        self._set_parameter("mik", value)

    @property
    def phone(self):
        return self._phone

    @phone.setter
    def phone(self, value):
        self._set_parameter("phone", value)

    @property
    def postal(self):
        return self._postal

    @postal.setter
    def postal(self, value):
        self._set_parameter("postal", value)

    @property
    def ssn(self):
        return self._ssn

    @ssn.setter
    def ssn(self, value):
        self._set_parameter("ssn", value)

    @property
    def state(self):
        return self._state

    @state.setter
    def state(self, value):
        self._set_parameter("state", value)

    @property
    def transmission_reference(self):
        return self._transmission_reference

    @transmission_reference.setter
    def transmission_reference(self, value):
        self._set_parameter("transmission_reference", value)

    @property
    def opt(self):
        return self._opt

    @opt.setter
    def opt(self, value):
        self._set_parameter("opt", value)

    @property
    def cols(self):
        return self._cols

    @cols.setter
    def cols(self, value):
        self._set_parameter("cols", value)

    # Setters
    def set_action(self, value):
        self._set_parameter("action", value)
    
    def set_address_line_1(self, value):
        self._set_parameter("address_line_1", value)
    
    def set_address_line_2(self, value):
        self._set_parameter("address_line_2", value)
    
    def set_birth_day(self, value):
        self._set_parameter("birth_day", value)
    
    def set_birth_month(self, value):
        self._set_parameter("birth_month", value)
    
    def set_birth_year(self, value):
        self._set_parameter("birth_year", value)
    
    def set_city(self, value):
        self._set_parameter("city", value)
    
    def set_company(self, value):
        self._set_parameter("company", value)
    
    def set_country(self, value):
        self._set_parameter("country", value)
    
    def set_email(self, value):
        self._set_parameter("email", value)
    
    def set_first_name(self, value):
        self._set_parameter("first_name", value)
    
    def set_free_form(self, value):
        self._set_parameter("free_form", value)
    
    def set_full_name(self, value):
        self._set_parameter("full_name", value)
    
    def set_ip(self, value):
        self._set_parameter("ip", value)
    
    def set_last_line(self, value):
        self._set_parameter("last_line", value)
    
    def set_last_name(self, value):
        self._set_parameter("last_name", value)
    
    def set_mak(self, value):
        self._set_parameter("mak", value)
    
    def set_mik(self, value):
        self._set_parameter("mik", value)
    
    def set_phone(self, value):
        self._set_parameter("phone", value)
    
    def set_postal(self, value):
        self._set_parameter("postal", value)
    
    def set_ssn(self, value):
        self._set_parameter("ssn", value)
    
    def set_state(self, value):
        self._set_parameter("state", value)
    
    def set_transmission_reference(self, value):
        self._set_parameter("transmission_reference", value)
    
    def set_opt(self, value):
        self._set_parameter("opt", value)
    
    def set_cols(self, value):
        self._set_parameter("cols", value)

    def set_post_body(self, post_body):
        self.post_body = post_body

    def set_value(self, parameter, value):
        parameter = parameter.strip() if parameter else None
        
        if parameter in self.parameter_mappings:
            parameter_key = self.parameter_mappings[parameter]
            
            property_name = next((key for key, val in self.parameter_mappings.items() 
                                  if val.lower() == parameter_key.lower()), None)
            
            if property_name:
                if hasattr(self, property_name):
                    setattr(self, property_name, value)
        else:
            # If not in derived class, go to base class implementation
            super().set_value(parameter, value)

    # Getters
    def get_action(self):
        return self.parameters.get("act", "")

    def get_address_line_1(self):
        return self.parameters.get("a1", "")

    def get_address_line_2(self):
        return self.parameters.get("a2", "")

    def get_birth_day(self):
        return self.parameters.get("bday", "")

    def get_birth_month(self):
        return self.parameters.get("bmo", "")

    def get_birth_year(self):
        return self.parameters.get("byr", "")

    def get_city(self):
        return self.parameters.get("city", "")

    def get_company(self):
        return self.parameters.get("comp", "")

    def get_country(self):
        return self.parameters.get("ctry", "")

    def get_email(self):
        return self.parameters.get("email", "")

    def get_first_name(self):
        return self.parameters.get("first", "")

    def get_free_form(self):
        return self.parameters.get("ff", "")

    def get_full_name(self):
        return self.parameters.get("full", "")

    def get_ip(self):
        return self.parameters.get("ip", "")

    def get_last_line(self):
        return self.parameters.get("lastline", "")

    def get_last_name(self):
        return self.parameters.get("last", "")

    def get_mak(self):
        return self.parameters.get("mak", "")

    def get_mik(self):
        return self.parameters.get("MIK", "")

    def get_phone(self):
        return self.parameters.get("phone", "")

    def get_postal(self):
        return self.parameters.get("postal", "")

    def get_ssn(self):
        return self.parameters.get("ss", "")

    def get_state(self):
        return self.parameters.get("state", "")

    def get_transmission_reference(self):
        return self.parameters.get("t", "")

    def get_opt(self):
        return self.parameters.get("opt", "")

    def get_cols(self):
        return self.parameters.get("cols", "")

    def get_post_body(self):
        return self.post_body

    # Class Methods
    def add_record(self, request):
        self.records.append(request)

    def clear_records(self):
        self.records.clear()

    def post(self, response_type):
        """
        This synchronous function makes the batch request using the post body.

        Returns:
            A string or deserialized json response object.
        """

        if self.records and len(self.records) > 0:
            post_request = PersonatorConsumerPostRequest(
                transmission_reference=self.get_transmission_reference(),
                options=self.get_opt(),
                records=self.records,
                customer_id=self.get_license()
            )
            response = self.send_post_request(post_request)
        else:
            response = self.send_post_request(self.post_body)

        return self.handle_response(response, response_type)

    def get_api_version(self):
        """
        Makes a synchronous getversion request and parses the response for the API version.

        Returns:
            The Cloud API version.
        """
        json_response = self.send_version_request()
        response_objectect = json.loads(json_response)
        version = response_objectect["Version"]
        return version
